//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//-- This confidential and proprietary software may be used only as authorized 
//-- by a licensing agreement from ESD University of Verona - EDALab s.r.l.                      
//--                                                                            
//--                                                                           
//-- Title:             Multimedia System Design Developement
//-- Project:           V-CLIP 1.0
//--    
//-- 
//-- File:              adpcm_UT.cpp
//-- 
//-- Mod History: $Log: $
//                                                                            
//----------------------------------------------------------------------------
//-- RCS Version:  $Id: $                                           
//----------------------------------------------------------------------------

#include "adpcm_UT.h"

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	1
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
		fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

// FUNCTIONALITY SIDE:
//****************
void adpcm_UT::adpcm_coder()
{

		int indexTable[16] = {
		    -1, -1, -1, -1, 2, 4, 6, 8,
		    -1, -1, -1, -1, 2, 4, 6, 8,
		};
		
		int stepsizeTable[89] = {
		    7, 8, 9, 10, 11, 12, 13, 14, 16, 17,
		    19, 21, 23, 25, 28, 31, 34, 37, 41, 45,
		    50, 55, 60, 66, 73, 80, 88, 97, 107, 118,
		    130, 143, 157, 173, 190, 209, 230, 253, 279, 307,
		    337, 371, 408, 449, 494, 544, 598, 658, 724, 796,
		    876, 963, 1060, 1166, 1282, 1411, 1552, 1707, 1878, 2066,
		    2272, 2499, 2749, 3024, 3327, 3660, 4026, 4428, 4871, 5358,
		    5894, 6484, 7132, 7845, 8630, 9493, 10442, 11487, 12635, 13899,
		    15289, 16818, 18500, 20350, 22385, 24623, 27086, 29794, 32767
		};
		
		c = 0;		
		outputbuffer = 0;
		val = 0;	
		sign = 0;	
		delta = 0;	
		step = 0;	
		vpdiff = 0;	
		len_value = 0; 
		valpred = 0;
		diff = 0;	
		index = 0;	
		bufferstep = 0;

		ioDataStruct.outdata_enable = false;
		len_value = ioDataStruct.len;
		    
		step = stepsizeTable[index];
		bufferstep = 1;

		val = ioDataStruct.indata;       // read new indata value
		diff = val - valpred;
		sign = (diff < 0) ? 8 : 0;
		if( sign ) 
			diff = (-diff);
		
		delta = 0;
		vpdiff = (step >> 3);
		if(diff >= (signed)step)
		{
			delta = 4;
			diff -= step;
			vpdiff += step;
		}

		step >>= 1;			
		if(diff >= (signed)step)
		{
			delta |= 2;
			diff -= step;
			vpdiff += step;
		}

		step >>= 1;			
		if(diff >= (signed)step)
		{
			delta |= 1;
			vpdiff += step;
		}

		if(sign)
			valpred -= vpdiff;
		else	
			valpred += vpdiff;

		if(valpred > 32767) 
			valpred = 32767;
		else if (valpred < -32768) 
			valpred = -32768;

		delta |= sign;
		index += indexTable[delta];
		if(index < 0) index = 0;
		if(index > 88) index = 88;
		step = stepsizeTable[index];

		len_value = len_value - 1;

		if (bufferstep)
			outputbuffer = (delta << 4) & 0xf0;
		else
		{
			c = (delta & 0x0f) | outputbuffer;
			ioDataStruct.outdata = c;
			ioDataStruct.outdata_enable = true;
		}
		bufferstep= !bufferstep;
		
		if (!bufferstep)
		{
			ioDataStruct.outdata = outputbuffer;
			ioDataStruct.outdata_enable = true;
		}
}

void adpcm_UT::reset()
{
  state = RESET;
}

void adpcm_UT::run()
{
	reset();
	
	//---------------------------
	read_test_input();
	int c = 0;
	//----------------------------
	while(true)
	{
		//---------------------------
		//---------------------------
		
		adpcm_coder();
		
		//---------------------------
		c++;
		if(c >= 10)
			break;
		//---------------------------
	}	
}
