//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//-- This confidential and proprietary software may be used only as authorized 
//-- by a licensing agreement from ESD University of Verona - EDALab s.r.l.                      
//--                                                                            
//--                                                                           
//-- Title:             Multimedia System Design Developement
//-- Project:           V-CLIP 1.0
//--    
//-- 
//-- File:              adpcm_RTL.h
//-- 
//-- Mod History: $Log: $
//                                                                            
//----------------------------------------------------------------------------
//-- RCS Version:  $Id: $                                           
//----------------------------------------------------------------------------

#include <systemc.h>

SC_MODULE(adpcm_RTL)
{
  // Input Port
  sc_in < sc_uint<16> > len;    // buffer size
  sc_in <short> indata;         // input value from buffer
  sc_in <bool> clk;             // system clock
  sc_in <bool> reset;           // reset for adpcm

  // Output Port
  sc_out <char> outdata;        // output values of adpcm
  sc_out <bool> outdata_enable; // 0->1 when a new value is computed and ready
  sc_out <bool> indata_enable;  // 0->1 when a new indata is required for continue
  sc_out <bool> len_enable;     // 0->1 when a new buffer is required for continue
	
  // States List
  enum state_adpcm {
    RESET,PRE1,PRE2,STEP01,STEP0,STEP1, STEP2, STEP3, STEP4, STEP5, STEP6
  };

  // Internal Signal
  sc_signal < state_adpcm > state;        // current state
  sc_signal < state_adpcm > next_state;   // next state
	
  // Internal Variable
  signed char c;		
  signed char outputbuffer;		/* place to keep previous 4-bit value */

  short val;			        /* Current input sample value */

  sc_uint<4> sign;			/* Current adpcm sign bit */
  sc_uint<4> delta;			/* Current adpcm output value */
  sc_uint<18> step;			/* Stepsize */
  sc_uint<18> vpdiff;			/* Current change to valpred */
  sc_uint<16> len_value;              	/* Buffer size in input*/	

  int valpred;			        /* Predicted output value */
  int diff;			        /* Difference between val and valprev */
  int index;			        /* Current step change index */

  bool bufferstep;		        /* toggle between outputbuffer/output */

  // Declare Processes
  void StateEvol();                     // FSM, state implementation
	
  // Constructor
  SC_CTOR (adpcm_RTL) {

    SC_METHOD (StateEvol);
    sensitive_pos << clk;
    
  }
  
};  // end SC_MODULE
