#include "systemc.h"
#include <stdlib.h>
#include <iostream>
#include "bcd_to_binary_L.h"

/////////////////////////////////////
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	1
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
		fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = atoi(subline.c_str());
				//std::cout << "i-" << test_data_in[c][i] << " " << atoi(subline.c_str());
			}
			c++;
			//std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = atoi(subline.c_str());
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}

////////////////////////////////////////////////
SC_MODULE(testt)
{
	sc_in<bool> clk_i;
	
	bcd_to_binary_L ins;
	sc_signal<sc_uint<5> > dat_bcd_i;
	
	sc_signal<sc_uint<4> > dat_bin_o;
	sc_signal<sc_uint<4> > dat_gray_o;
	
	void test()
	{
		std::cout << "start simulation " << std::endl;

		int c = 0;
		while(true)
		{
			dat_bcd_i.write(test_data_in[c][0]);
			std::cout << sc_time_stamp() << "i-" << dat_bcd_i.read() << std::endl;
			
			wait(clk_i.negedge_event());
			
			std::cout << sc_time_stamp() << "o-" << dat_bin_o.read() << " " << dat_gray_o.read() << std::endl;
			
			if(test_data_out_x[c][0] == 0)
				assert((unsigned)(dat_bin_o.read()) == test_data_out[c][0]);
			if(test_data_out_x[c][1] == 0)
				assert((unsigned)(dat_gray_o.read()) == test_data_out[c][1]);
			c++;
		}	
		std::cout << "start simulation " << std::endl;
	}
	
	SC_CTOR(testt):ins("ins")
	{
		//SC_CTHREAD(test, clk_i.pos());
		SC_THREAD(test);
		
		ins.clk_i(clk_i);
		ins.dat_bcd_i(dat_bcd_i);
		ins.dat_bin_o(dat_bin_o);
		ins.dat_gray_o(dat_gray_o);
	}
};

///////////////////////////////////////
int sc_main(int argc, char *argv[])
{
	read_test_input();
	
	// start simulation
	sc_clock clk;
	testt tester("tester");
	tester.clk_i(clk);
	sc_start(10,SC_NS);
	return 0;
}
