#include "systemc.h"
#include <stdlib.h>
#include <iostream>
#include "byte_mixcolum_L.h"

#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	4
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

SC_MODULE(testt)
{
	sc_in<bool> clk;
	
	byte_mixcolum_L ins;
	sc_signal<sc_uint<8> > a;
	sc_signal<sc_uint<8> > b;
	sc_signal<sc_uint<8> > c;
	sc_signal<sc_uint<8> > d;
	
	sc_signal<sc_uint<8> > outx;
	sc_signal<sc_uint<8> > outy;
	
	void test()
	{
		std::cout << "start simulation " << std::endl;
		
		int c2 = 0;
		while(true)
		{
			a.write(test_data_in[c2][0]);
			b.write(test_data_in[c2][1]);
			c.write(test_data_in[c2][2]);
			d.write(test_data_in[c2][3]);
			std::cout << sc_time_stamp() << "i-" << a.read() << " " << b.read() << " " << c.read() << " " << d.read() << std::endl;
			
			wait(clk.negedge_event());
			
			std::cout << sc_time_stamp() << "o-" << outx.read() << " " << outy.read() << std::endl;
			
			if(test_data_out_x[c2][0] == 0)
				assert((unsigned)(outx.read()) == test_data_out[c2][0]);
			if(test_data_out_x[c2][1] == 0)
				assert((unsigned)(outy.read()) == test_data_out[c2][1]);
			c2++;
		}	
		std::cout << "end simulation " << std::endl;
	}
	
	SC_CTOR(testt):ins("ins")
	{
		SC_THREAD(test);
		
		ins.a(a);
		ins.b(b);
		ins.c(c);
		ins.d(d);
		ins.outx(outx);
		ins.outy(outy);
	}
};

int sc_main(int argc, char *argv[])
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	sc_uint<16> val;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = atoi(subline.c_str());
				//std::cout << "i-" << test_data_in[c][i] << " " << atoi(subline.c_str());
			}
			c++;
			//std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = atoi(subline.c_str());
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			std::cout << "else: " << line << std::endl;
			assert(0);
		}
	}
	
	// start simulation
	sc_clock clk;
	testt tester("tester");
	tester.clk(clk);
	sc_start(10,SC_NS);
	return 0;
}
