#include <iostream>
#include "systemc.h"

// BINARY ENCODED state machine: drctCntl
// State codes definitions:
#define START_DC 0
#define CHK_DRCT_CNTL 1
#define DRCT_CNTL_WAIT_GNT 2
#define DRCT_CNTL_CHK_LOOP 3
#define DRCT_CNTL_WAIT_RDY 4
#define IDLE_FIN 5
#define IDLE_WAIT_GNT 6
#define IDLE_WAIT_RDY 7

#define TX_DIRECT_CONTROL 0
#define TX_IDLE 5

SC_MODULE(directControl_H)
{
  bool		HCTxPortGnt;
  bool		HCTxPortRdy;
  bool		directControlEn;
  unsigned	directControlLineState;
  bool		rst;

  unsigned	HCTxPortCntl;
  unsigned	HCTxPortData;
  bool		HCTxPortReq;
  bool		HCTxPortWEn;

  void run();
  void drtCntl(unsigned);

  unsigned	state_drctCntl;
	
  SC_CTOR(directControl_H)
  {
    SC_THREAD(run);
  }
};

void directControl_H::drtCntl(unsigned state)
{
  switch (state)
  {
    case START_DC:
      state_drctCntl = CHK_DRCT_CNTL;
      break;
    case CHK_DRCT_CNTL:
      if (directControlEn == true)	
      {
        state_drctCntl = DRCT_CNTL_WAIT_GNT;
      }
      else
      {
        state_drctCntl = IDLE_WAIT_GNT;
      }
      HCTxPortReq = true;
      break;
    case DRCT_CNTL_WAIT_GNT:
      if (HCTxPortGnt == true)	
      {
        state_drctCntl = DRCT_CNTL_WAIT_RDY;
      }
      break;
    case DRCT_CNTL_CHK_LOOP:
      HCTxPortWEn = false;
      if (directControlEn == false)	
      {
        state_drctCntl = CHK_DRCT_CNTL;
        HCTxPortReq = false;
      }
      else
      {
        state_drctCntl = DRCT_CNTL_WAIT_RDY;
      }
      break;
    case DRCT_CNTL_WAIT_RDY:
      if (HCTxPortRdy == true)	
      {
        state_drctCntl = DRCT_CNTL_CHK_LOOP;
        HCTxPortWEn = true;
//         next_HCTxPortData <= {6'b000000, directControlLineState};
        HCTxPortData = directControlLineState;
        HCTxPortCntl = TX_DIRECT_CONTROL;
      }
      break;
    case IDLE_FIN:
      HCTxPortWEn = false;
      HCTxPortReq = false;
      state_drctCntl = CHK_DRCT_CNTL;
      break;
    case IDLE_WAIT_GNT:
      if (HCTxPortGnt == true)	
        state_drctCntl = IDLE_WAIT_RDY;
      break;
    case IDLE_WAIT_RDY:
      if (HCTxPortRdy == true)	
      {
        state_drctCntl = IDLE_FIN;
        HCTxPortWEn = true;
        HCTxPortData = 0;
        HCTxPortCntl = TX_IDLE;
      }
      break;
  }
}

//--------------------------------------------------------------------
// Machine: drctCntl
//--------------------------------------------------------------------
void directControl_H::run()
{
  HCTxPortGnt = false;
  HCTxPortRdy = false;
  directControlEn = false;
  rst = true;
  directControlLineState = 10;

  while(true)
  {
    if (rst)
    {
      state_drctCntl = START_DC;
      HCTxPortCntl = 0;
      HCTxPortData = 0;
      HCTxPortWEn = 0;
      HCTxPortReq = 0;
    }
    else
      drtCntl(state_drctCntl);
  }
}

