module fft(in_real, in_imag, data_valid, data_ack, CLK, out_real, out_imag, data_req, data_ready);
input [16:0] in_real;
input [16:0] in_imag;
input data_valid;
input data_ack;
input CLK;

output [16:0] out_real;
output [16:0] out_imag;
output data_req;
output data_ready;

reg [15:0] sample_real[0:15];
reg [15:0] sample_imag[0:15];

reg [16:0] out_real;
reg [16:0] out_imag;
reg data_req;
reg data_ready;

wire [4:0] N;
wire [4:0] M;
reg [4:0] len;
reg [15:0] w_real;
reg [15:0] w_imag;
reg [15:0] w_rec_real;
reg [15:0] w_rec_imag;
reg [31:0] w_temp;

reg [15:0] W_real[0:6];
reg [15:0] W_imag[0:6];

reg [15:0] incr;
reg [15:0] stage;
reg [15:0] i;
reg [15:0] index;
reg [15:0] index2;
reg [15:0] windex;
reg [15:0] j;

reg [15:0] tmp_real;
reg [15:0] tmp_imag;
reg [15:0] tmp_real2;
reg [15:0] tmp_imag2;

reg[3:0] bits_i;
reg[3:0] bits_index;

always @ (posedge CLK)
begin
	data_req <= 0;
	data_ready <= 0;

	for(index = 0; index < 16; index = index+1) begin
		data_req <= 1;
		wait(data_valid == 0)
		begin
			sample_real[index] = in_real;
			sample_imag[index] = in_imag;
		end
		data_req <= 0;
	end
end

assign	M = 4;
assign	N = 16;

always @ (posedge CLK)
begin
	len = N >> 1;
	w_real = 942;
	w_imag = -389;
	w_rec_real = 1;
	w_rec_imag = 0;

	for(index = 0; index < len-1; index = index+1) begin
		w_temp = w_rec_real*w_real - w_rec_imag*w_imag;
		w_rec_imag = w_rec_real*w_imag + w_rec_imag*w_real;
		w_rec_real = w_temp;
		W_real[index] = w_rec_real;
		W_imag[index] = w_rec_imag;
	end
end

always @ (posedge CLK)
begin
	len = N;
	incr = 1;

	for(stage = 0; stage < N; stage = stage+1) begin
		len = len >> 1;

		for(i=0; i < N; i=i+2*len) begin
			index = i;
			index2 = index + len;

			tmp_real = sample_real[index] + sample_real[index2];
			tmp_imag = sample_imag[index] + sample_imag[index2];

			sample_real[index2] = sample_real[index] - sample_real[index2];
			sample_imag[index2] = sample_imag[index] - sample_imag[index2];

			sample_real[index] = tmp_real;
			sample_imag[index] = tmp_imag;
		end

		windex = incr - 1;
		for(j=1; j < len; j=j+1) begin
			for(i = j; i < N; i = i+2*len) begin
				index = i;
				index2 = index + len;

				tmp_real = sample_real[index] + sample_real[index2];
				tmp_imag = sample_imag[index] + sample_imag[index2];
				tmp_real2 = sample_real[index] - sample_real[index2];
				tmp_imag2 = sample_imag[index] - sample_imag[index2];

				sample_real[index2] = tmp_real2*W_real[windex]- tmp_imag2*W_imag[windex];
				sample_imag[index2] = tmp_real2*W_imag[windex]+ tmp_imag2*W_real[windex];

				sample_real[index] = tmp_real;
				sample_imag[index] = tmp_imag;
			end

			windex = windex + incr;
		end
		
		incr = 2*incr;
	end
end

always @ (posedge CLK)
begin
	bits_i = 0;
	for(i = 0; i < 16; i = i+1) begin
		bits_i = i;
		bits_index = {bits_i[0], bits_i[1],bits_i[2],bits_i[3]};
		index = bits_index;

		out_real = sample_real[index];
		out_imag = sample_imag[index];
		data_ready <= 1;
		wait(data_ack == 1) begin
			data_ready <= 0;
		end
	end
end

endmodule