`include "hostSlaveMux.v"

module top();

  reg [7:0] SIEPortCtrlInFromHost;
  reg [7:0] SIEPortCtrlInFromSlave;
  reg [7:0] SIEPortDataInFromHost; 
  reg [7:0] SIEPortDataInFromSlave; 
  reg SIEPortWEnFromHost; 
  reg SIEPortWEnFromSlave; 
  reg fullSpeedPolarityFromHost;
  reg fullSpeedPolarityFromSlave;
  reg fullSpeedBitRateFromHost;
  reg fullSpeedBitRateFromSlave;
  reg noActivityTimeOutEnableFromHost;
  reg noActivityTimeOutEnableFromSlave;
  //hostSlaveMuxBI
  reg [7:0] dataIn;
  reg address;
  reg writeEn;
  reg strobe_i;
  reg busClk;
  reg usbClk;
  reg rstFromWire;
  reg hostSlaveMuxSel;

  wire [7:0] SIEPortCtrlInToSIE;
  wire [7:0] SIEPortDataInToSIE; 
  wire SIEPortWEnToSIE; 
  wire fullSpeedPolarityToSIE;
  wire fullSpeedBitRateToSIE;
  wire noActivityTimeOutEnableToSIE;
  wire rstSyncToBusClkOut;
  wire rstSyncToUsbClkOut;
  wire [7:0] dataOut;

// Clock generator
always begin
  #5 
  usbClk = ~usbClk;
  busClk = ~busClk;
end

initial begin
  $dumpfile ("hostSlaveMux.vcd");
  $dumpvars();
  usbClk <= 0;
  busClk <= 1;
  rstFromWire <= 1;

  SIEPortCtrlInFromHost <= 3;
  SIEPortCtrlInFromSlave <= 4;
  SIEPortDataInFromHost <= 5;
  SIEPortDataInFromSlave <= 6;

  SIEPortWEnFromHost <= 0;
  SIEPortWEnFromSlave <= 0;
  fullSpeedPolarityFromHost <= 0;
  fullSpeedPolarityFromSlave <= 0;
  fullSpeedBitRateFromHost <= 0;
  fullSpeedBitRateFromSlave <= 0;
  noActivityTimeOutEnableFromHost <= 0;
  noActivityTimeOutEnableFromSlave <= 0;

  dataIn <= 7; 

  address <= 0; 
  writeEn <= 0; 
  strobe_i <= 0; 
  hostSlaveMuxSel <= 0; 

  #10 
  rstFromWire <= 0; 

  #10
  SIEPortWEnFromHost <= 1; 
  fullSpeedPolarityFromHost <= 1; 
  fullSpeedBitRateFromHost <= 1; 
  noActivityTimeOutEnableFromHost <= 1; 

  #10
  SIEPortWEnFromSlave <= 1; 
  fullSpeedPolarityFromSlave <= 1; 
  fullSpeedBitRateFromSlave <= 1; 
  noActivityTimeOutEnableFromSlave <= 1; 

  #10
  SIEPortWEnFromHost <= 0; 
  fullSpeedPolarityFromHost <= 0; 
  fullSpeedBitRateFromHost <= 0; 
  noActivityTimeOutEnableFromHost <= 0; 

  #10
  address <= 1; 
  writeEn <= 1; 

  #10
  strobe_i <= 1; 
  rstFromWire <= 1; 
  hostSlaveMuxSel <= 1; 

  #10
  writeEn <= 0; 

  #10
  strobe_i <= 0; 
  rstFromWire <= 0; 

  #10
  hostSlaveMuxSel <= 0; 

  #20 $finish;
end 

// Connect the DUT
hostSlaveMux U (
 SIEPortCtrlInToSIE,
  SIEPortCtrlInFromHost,
  SIEPortCtrlInFromSlave,
  SIEPortDataInToSIE, 
  SIEPortDataInFromHost, 
  SIEPortDataInFromSlave, 
  SIEPortWEnToSIE, 
  SIEPortWEnFromHost, 
  SIEPortWEnFromSlave, 
  fullSpeedPolarityToSIE,
  fullSpeedPolarityFromHost,
  fullSpeedPolarityFromSlave,
  fullSpeedBitRateToSIE,
  fullSpeedBitRateFromHost,
  fullSpeedBitRateFromSlave,
  noActivityTimeOutEnableToSIE,
  noActivityTimeOutEnableFromHost,
  noActivityTimeOutEnableFromSlave,
  dataIn, 
  dataOut,
  address,
  writeEn,
  strobe_i,
  busClk, 
  usbClk, 
  hostSlaveMuxSel,
  rstFromWire,
  rstSyncToBusClkOut,
  rstSyncToUsbClkOut
);

always @ (posedge busClk or posedge usbClk)
begin 
  $display ("%d %d %d %d %d %b %b %b %b %b %b %b %b %d %b %b %b %b %b %b %b \t %d %d %b %b %b %b %b %b %d", $time, 
  SIEPortCtrlInFromHost,
  SIEPortCtrlInFromSlave,
  SIEPortDataInFromHost, 
  SIEPortDataInFromSlave, 
  SIEPortWEnFromHost, 
  SIEPortWEnFromSlave, 
  fullSpeedPolarityFromHost,
  fullSpeedPolarityFromSlave,
  fullSpeedBitRateFromHost,
  fullSpeedBitRateFromSlave,
  noActivityTimeOutEnableFromHost,
  noActivityTimeOutEnableFromSlave,
  dataIn, 
  address,
  writeEn,
  strobe_i,
  busClk, 
  usbClk, 
  hostSlaveMuxSel,
  rstFromWire,
  SIEPortCtrlInToSIE,
  SIEPortDataInToSIE, 
  SIEPortWEnToSIE, 
  fullSpeedPolarityToSIE,
  fullSpeedBitRateToSIE,
  noActivityTimeOutEnableToSIE,
  rstSyncToBusClkOut,
  rstSyncToUsbClkOut,
  dataOut
  );
end

endmodule
