#include "systemc.h"

SC_MODULE(hostSlaveMuxBI_H)
{
  sc_uint<8> dataIn;
  bool address;
  bool writeEn;
  bool strobe_i;
  bool hostSlaveMuxSel;
  bool rstFromWire;

  sc_uint<8> dataOut;
  bool hostMode;
  bool rstSyncToBusClkOut;
  bool rstSyncToUsbClkOut;

  void sync_write_demux();
  void async_read_mux(bool);
  void reset_control();
  void rst_shift();
  void double_sync();
  void show();
  void run();

  //internal wire and regs
  sc_uint<6> rstShift;
  bool rstFromBus;
  bool rstSyncToUsbClkFirst;
	
  SC_CTOR(hostSlaveMuxBI_H)
  {
    SC_THREAD(run);
  }
};

void hostSlaveMuxBI_H::show()
{
  cout << dataIn << " " << address << " " << writeEn << " " << strobe_i << " " 
	<< hostSlaveMuxSel << " " << rstFromWire << " \t"
	<< dataOut << " " << hostMode << " " << rstSyncToBusClkOut << " " << rstSyncToUsbClkOut
	<< endl;
}

//sync write demux
void hostSlaveMuxBI_H::sync_write_demux()
{
  if (rstSyncToBusClkOut == true)
  {
    hostMode = false;
  }
  else 
  {
    if (writeEn == true && hostSlaveMuxSel == true && 
      strobe_i == true && address == HOST_SLAVE_CONTROL_REG)
    {
      hostMode = dataIn[0]; 
    }
    
    if (writeEn == true && hostSlaveMuxSel == true && 
      strobe_i == true && address == HOST_SLAVE_CONTROL_REG && (dataIn[1]) == true)
    {
      rstFromBus = true; 
    }
    else
    {
      rstFromBus = false; 
    }
  }
}

// async read mux
void hostSlaveMuxBI_H::async_read_mux(bool addr)
{
  switch(addr)
  {
    case HOST_SLAVE_CONTROL_REG:
      dataOut = 0;
      dataOut[0] = hostMode; 
      break;
    case HOST_SLAVE_VERSION_REG:
      dataOut = USBHOSTSLAVE_VERSION_NUM; 
      break;
  }
}

// reset control
//generate 'rstSyncToBusClk'
//assuming that 'busClk' < 5 * 'usbClk'. ie 'busClk' < 240MHz
void hostSlaveMuxBI_H::reset_control()
{
  if ((rstFromWire == true) || (rstFromBus == true))
  {
    rstShift = 0x3F; 
  }
  else
  {
    rstShift[5] = 0;
    rstShift.range(4, 0) = rstShift.range(5, 1); 
  }
}

void hostSlaveMuxBI_H::rst_shift()
{
  rstSyncToBusClkOut = rstShift[0]; 
}

// double sync across clock domains to generate 'forceEmptySyncToWrClk'
void hostSlaveMuxBI_H::double_sync()
{
  rstSyncToUsbClkFirst = rstSyncToBusClkOut; 
  rstSyncToUsbClkOut = rstSyncToUsbClkFirst; 
}

void hostSlaveMuxBI_H::run()
{
  dataIn = 10;
  address = false;
  writeEn = false;
  strobe_i = false;
  hostSlaveMuxSel = true;
  rstFromWire = true;

  while(true)
  {
    show();
    sync_write_demux();
    async_read_mux(address);
    reset_control();
    rst_shift();
    double_sync();
  }
}

