#include "systemc.h"

SC_MODULE(hostSlaveMuxBI_L)
{
  sc_in<sc_uint<8> > dataIn;
  sc_in<bool> address;
  sc_in<bool> writeEn;
  sc_in<bool> strobe_i;
  sc_in<bool> busClk;
  sc_in<bool> usbClk;
  sc_out<sc_uint<8> > dataOut;
  sc_in<bool> hostSlaveMuxSel;
  sc_out<bool> hostMode;
  sc_in<bool> rstFromWire;
  sc_out<bool> rstSyncToBusClkOut;
  sc_out<bool> rstSyncToUsbClkOut;

  void sync_write_demux();
  void async_read_mux();
  void reset_control();
  void rst_shift();
  void double_sync();
	
  //internal wire and regs
  sc_signal<sc_uint<6> > rstShift;
  sc_signal<bool> rstFromBus;
  sc_signal<bool> rstSyncToUsbClkFirst;

  SC_CTOR(hostSlaveMuxBI_L)
  {
    SC_METHOD(sync_write_demux);
    sensitive << busClk.pos();

    SC_METHOD(async_read_mux);
    sensitive << address << hostMode;

    SC_METHOD(reset_control);
    sensitive << busClk.pos();

    SC_METHOD(rst_shift);
    sensitive << rstShift;

    SC_METHOD(double_sync);
    sensitive << usbClk.pos();
  }
};

#include <iostream>
#include "hostSlaveMuxBI_L.h"

//sync write demux
void hostSlaveMuxBI_L::sync_write_demux()
{
  if (rstSyncToBusClkOut.read() == true)
  {
    hostMode.write(false);
  }
  else 
  {
    if (writeEn.read() == true && hostSlaveMuxSel.read() == true && 
      strobe_i.read() == true && address.read() == HOST_SLAVE_CONTROL_REG)
    {
      hostMode.write(dataIn.read()[0]);
    }
    
    if (writeEn.read() == true && hostSlaveMuxSel.read() == true && 
      strobe_i.read() == true && address.read() == HOST_SLAVE_CONTROL_REG && (dataIn.read()[1]) == true)
    {
      rstFromBus.write(true);
    }
    else
    {
      rstFromBus.write(false);
    }
  }
}

// async read mux
void hostSlaveMuxBI_L::async_read_mux()
{
  switch(address.read())
  {
    case HOST_SLAVE_CONTROL_REG:
      dataOut.write(0x00 | hostMode.read());
      break;
    case HOST_SLAVE_VERSION_REG:
      dataOut.write(USBHOSTSLAVE_VERSION_NUM);
      break;
  }
}

// reset control
//generate 'rstSyncToBusClk'
//assuming that 'busClk' < 5 * 'usbClk'. ie 'busClk' < 240MHz
void hostSlaveMuxBI_L::reset_control()
{
  if ((rstFromWire.read() == true) || (rstFromBus.read() == true))
  {
    rstShift.write(0x3F);
  }
  else
  {
    rstShift.write(0x00 | (rstShift.read()).range(5, 1));
  }
}

void hostSlaveMuxBI_L::rst_shift()
{
  rstSyncToBusClkOut.write(rstShift.read()[0]);
}

// double sync across clock domains to generate 'forceEmptySyncToWrClk'
void hostSlaveMuxBI_L::double_sync()
{
  rstSyncToUsbClkFirst.write(rstSyncToBusClkOut.read());
  rstSyncToUsbClkOut.write(rstSyncToUsbClkFirst.read());
}


