#include "systemc.h"
#include "hostSlaveMuxBI_L.h"

SC_MODULE(hostSlaveMux_L)
{
  sc_out<sc_uint<8> > SIEPortCtrlInToSIE;
  sc_in<sc_uint<8> > SIEPortCtrlInFromHost;
  sc_in<sc_uint<8> > SIEPortCtrlInFromSlave;
  sc_out<sc_uint<8> > SIEPortDataInToSIE; 
  sc_in<sc_uint<8> > SIEPortDataInFromHost; 
  sc_in<sc_uint<8> > SIEPortDataInFromSlave; 
  sc_out<bool> SIEPortWEnToSIE; 
  sc_in<bool> SIEPortWEnFromHost; 
  sc_in<bool> SIEPortWEnFromSlave; 
  sc_out<bool> fullSpeedPolarityToSIE;
  sc_in<bool> fullSpeedPolarityFromHost;
  sc_in<bool> fullSpeedPolarityFromSlave;
  sc_out<bool> fullSpeedBitRateToSIE;
  sc_in<bool> fullSpeedBitRateFromHost;
  sc_in<bool> fullSpeedBitRateFromSlave;
  sc_out<bool> noActivityTimeOutEnableToSIE;
  sc_in<bool> noActivityTimeOutEnableFromHost;
  sc_in<bool> noActivityTimeOutEnableFromSlave;
 
 //hostSlaveMuxBI
  sc_in<sc_uint<8> > dataIn;
  sc_in<bool> address;
  sc_in<bool> writeEn;
  sc_in<bool> strobe_i;
  sc_in<bool> busClk;
  sc_in<bool> usbClk;
  sc_in<bool> rstFromWire;
  sc_out<bool> rstSyncToBusClkOut;
  sc_out<bool> rstSyncToUsbClkOut;
  sc_out<sc_uint<8> > dataOut;
  sc_in<bool> hostSlaveMuxSel;

  void host_slave_mux();

  //internal wires and regs
  sc_signal<bool> hostMode;
  hostSlaveMuxBI_L *ins;

  SC_CTOR(hostSlaveMux_L)
  {
    SC_METHOD(host_slave_mux);
    sensitive << hostMode << SIEPortCtrlInFromHost <<  SIEPortCtrlInFromSlave <<
	      SIEPortDataInFromHost << SIEPortDataInFromSlave << SIEPortWEnFromHost <<
	      SIEPortWEnFromSlave << fullSpeedPolarityFromHost << fullSpeedPolarityFromSlave <<
	      fullSpeedBitRateFromHost << fullSpeedBitRateFromSlave << noActivityTimeOutEnableFromHost <<
	      noActivityTimeOutEnableFromSlave;

    ins = new hostSlaveMuxBI_L("hostSlaveMuxBI_L");
    
    ins->dataIn(dataIn);
    ins->dataOut(dataOut);
    ins->address(address);
    ins->writeEn(writeEn);
    ins->strobe_i(strobe_i);
    ins->busClk(busClk); 
    ins->usbClk(usbClk);
    ins->hostMode(hostMode);
    ins->hostSlaveMuxSel(hostSlaveMuxSel);  
    ins->rstFromWire(rstFromWire);
    ins->rstSyncToBusClkOut(rstSyncToBusClkOut);
    ins->rstSyncToUsbClkOut(rstSyncToUsbClkOut);
  }
};

void hostSlaveMux_L::host_slave_mux()
{
  if (hostMode.read() == true) 
  {
    SIEPortCtrlInToSIE.write(SIEPortCtrlInFromHost.read());
    SIEPortDataInToSIE.write( SIEPortDataInFromHost.read());
    SIEPortWEnToSIE.write(SIEPortWEnFromHost.read());
    fullSpeedPolarityToSIE.write(fullSpeedPolarityFromHost.read());
    fullSpeedBitRateToSIE.write(fullSpeedBitRateFromHost.read());
    noActivityTimeOutEnableToSIE.write(noActivityTimeOutEnableFromHost.read());
  }
  else
  {
    SIEPortCtrlInToSIE.write(SIEPortCtrlInFromSlave.read());
    SIEPortDataInToSIE.write( SIEPortDataInFromSlave.read());
    SIEPortWEnToSIE.write(SIEPortWEnFromSlave.read());
    fullSpeedPolarityToSIE.write(fullSpeedPolarityFromSlave.read());
    fullSpeedBitRateToSIE.write(fullSpeedBitRateFromSlave.read());
    noActivityTimeOutEnableToSIE.write(noActivityTimeOutEnableFromSlave.read());
  }
}


