#include <iostream>
#include "hostSlaveMuxBI_H.h"

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	6
#define NUM_OUT 4

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

//sync write demux
void hostSlaveMuxBI_H::sync_write_demux()
{
  if (rstSyncToBusClkOut == true)
  {
    hostMode = false;
  }
  else 
  {
    if (writeEn == true && hostSlaveMuxSel == true && 
      strobe_i == true && address == HOST_SLAVE_CONTROL_REG)
    {
      hostMode = dataIn[0]; 
    }
    
    if (writeEn == true && hostSlaveMuxSel == true && 
      strobe_i == true && address == HOST_SLAVE_CONTROL_REG && (dataIn[1]) == true)
    {
      rstFromBus = true; 
    }
    else
    {
      rstFromBus = false; 
    }
  }
}

// async read mux
void hostSlaveMuxBI_H::async_read_mux(bool addr)
{
  switch(addr)
  {
    case HOST_SLAVE_CONTROL_REG:
      dataOut = 0;
      dataOut[0] = hostMode; 
      break;
    case HOST_SLAVE_VERSION_REG:
      dataOut = USBHOSTSLAVE_VERSION_NUM; 
      break;
  }
}

// reset control
//generate 'rstSyncToBusClk'
//assuming that 'busClk' < 5 * 'usbClk'. ie 'busClk' < 240MHz
void hostSlaveMuxBI_H::reset_control()
{
  if ((rstFromWire == true) || (rstFromBus == true))
  {
    rstShift = 0x3F; 
  }
  else
  {
    rstShift[5] = 0;
    rstShift.range(4, 0) = rstShift.range(5, 1); 
  }
}

void hostSlaveMuxBI_H::rst_shift()
{
  rstSyncToBusClkOut = rstShift[0]; 
}

// double sync across clock domains to generate 'forceEmptySyncToWrClk'
void hostSlaveMuxBI_H::double_sync()
{
  rstSyncToUsbClkFirst = rstSyncToBusClkOut; 
  rstSyncToUsbClkOut = rstSyncToUsbClkFirst; 
}

void hostSlaveMuxBI_H::run()
{
//   dataIn = 10;
//   address = false;
//   writeEn = false;
//   strobe_i = false;
//   hostSlaveMuxSel = true;
//   rstFromWire = true;
	//---------------------------
	read_test_input();
	int c = 0;
	//----------------------------

  while(true)
  {
		//-------------------------
		dataIn = test_data_in[c][0];
		address = test_data_in[c][1];
		writeEn = test_data_in[c][2];
		strobe_i = test_data_in[c][3];
		hostSlaveMuxSel = test_data_in[c][4];
		rstFromWire = test_data_in[c][5];
		std::cout << sc_time_stamp() << "i-"
		<< dataIn << " "
		<< address << " "
		<< writeEn << " "
		<< strobe_i << " "
		<< hostSlaveMuxSel << " "
		<< rstFromWire << " "
		<< std::endl;
		//-------------------------
		
		sync_write_demux();
    async_read_mux(address);
    reset_control();
    rst_shift();
    double_sync();
		
		//-------------------------
		std::cout << sc_time_stamp() << "o-"
		<< (unsigned)dataOut << " "
		<< hostMode << " "
		<< rstSyncToBusClkOut << " "
		<< rstSyncToUsbClkOut << " "
		<< std::endl;
		if(test_data_out_x[c][0] == 0)
			assert((unsigned)(dataOut) == test_data_out[c][0]);
		if(test_data_out_x[c][1] == 0)
			assert((unsigned)(hostMode) == test_data_out[c][1]);
		if(test_data_out_x[c][0] == 0)
			assert((unsigned)(rstSyncToBusClkOut) == test_data_out[c][2]);
		if(test_data_out_x[c][1] == 0)
			assert((unsigned)(rstSyncToUsbClkOut) == test_data_out[c][3]);
		c++;
		if(c >= 9)	break;
		//-------------------------
  }
}


