#include "systemc.h"
#include <iostream>
#include "hostSlaveMuxBI_L.h"

SC_MODULE(testt)
{
	sc_out<sc_uint<8> > dataIn;
	sc_out<bool> address;
	sc_out<bool> writeEn;
	sc_out<bool> strobe_i;
	sc_out<bool> busClk;
	sc_out<bool> usbClk;
	sc_out<bool> hostSlaveMuxSel;
	sc_out<bool> rstFromWire;

	void stimuli()
	{
		dataIn.write(7);
		address.write(false);
		writeEn.write(false);
		strobe_i.write(false);
		hostSlaveMuxSel.write(false);
		rstFromWire.write(false);

		wait(10, SC_NS); 
		address.write(true);
		writeEn.write(true);
		strobe_i.write(true);
		hostSlaveMuxSel.write(true);
		rstFromWire.write(true);

		wait(10, SC_NS); 
		address.write(false);
		writeEn.write(false);

		wait(10, SC_NS); 
		strobe_i.write(false);

		wait(10, SC_NS); 
		hostSlaveMuxSel.write(false);
		rstFromWire.write(false);

		wait(10, SC_NS); 
		writeEn.write(true);

		wait(10, SC_NS); 
		address.write(true);

		wait(10, SC_NS); 
		hostSlaveMuxSel.write(true);

		wait(20, SC_NS); 
		sc_stop();
	}

	void clk_gen()
	{
		while(true) 
		{
			busClk.write(false);
			usbClk.write(true);
			wait(5, SC_NS);

			busClk.write(true);
			usbClk.write(false);
			wait(5, SC_NS);
		}
	}

	SC_CTOR(testt) 
	{
		SC_THREAD(stimuli);
		SC_THREAD(clk_gen);
	}
};

SC_MODULE(monitor)
{
  sc_in<sc_uint<8> > dataIn;
  sc_in<bool> address;
  sc_in<bool> writeEn;
  sc_in<bool> strobe_i;
  sc_in<bool> busClk;
  sc_in<bool> usbClk;
  sc_in<sc_uint<8> > dataOut;
  sc_in<bool> hostSlaveMuxSel;
  sc_in<bool> hostMode;
  sc_in<bool> rstFromWire;
  sc_in<bool> rstSyncToBusClkOut;
  sc_in<bool> rstSyncToUsbClkOut;

  void MonGen()
  {
    std::cout << sc_time_stamp() << " " << dataIn << " " << address << " " 
	    << writeEn << " " << strobe_i << " " << busClk << " " << usbClk << " " 
	    << hostSlaveMuxSel << " " << rstFromWire << " \t"
	    << dataOut << " " << hostMode << " " << rstSyncToBusClkOut << " " << rstSyncToUsbClkOut
	    << std::endl;
  }

  SC_CTOR(monitor)
  {
    SC_METHOD(MonGen);
    sensitive << usbClk.pos() << busClk.pos();
    //dont_initialize();
  }
};

// here it goes.
int sc_main(int argc, char *argv[])
{  
	sc_signal<sc_uint<8> > dataIn;
  sc_signal<bool> address;
  sc_signal<bool> writeEn;
  sc_signal<bool> strobe_i;
  sc_signal<bool> busClk;
  sc_signal<bool> usbClk;
  sc_signal<sc_uint<8> > dataOut;
  sc_signal<bool> hostSlaveMuxSel;
  sc_signal<bool> hostMode;
  sc_signal<bool> rstFromWire;
  sc_signal<bool> rstSyncToBusClkOut;
  sc_signal<bool> rstSyncToUsbClkOut;

  sc_set_time_resolution(1, SC_NS);

	hostSlaveMuxBI_L ins("ins");
	ins.dataIn(dataIn);
	ins.address(address);
	ins.writeEn(writeEn);
	ins.strobe_i(strobe_i);
	ins.busClk(busClk);
	ins.usbClk(usbClk);
	ins.hostSlaveMuxSel(hostSlaveMuxSel);
	ins.rstFromWire(rstFromWire);
	ins.dataOut(dataOut);
	ins.hostMode(hostMode);
	ins.rstSyncToBusClkOut(rstSyncToBusClkOut);
	ins.rstSyncToUsbClkOut(rstSyncToUsbClkOut);

	testt tb("tb");
	tb.dataIn(dataIn);
	tb.address(address);
	tb.writeEn(writeEn);
	tb.strobe_i(strobe_i);
	tb.busClk(busClk);
	tb.usbClk(usbClk);
	tb.hostSlaveMuxSel(hostSlaveMuxSel);
	tb.rstFromWire(rstFromWire);

	monitor mon("monitor");
	mon.dataIn(dataIn);
	mon.address(address);
	mon.writeEn(writeEn);
	mon.strobe_i(strobe_i);
	mon.busClk(busClk);
	mon.usbClk(usbClk);
	mon.hostSlaveMuxSel(hostSlaveMuxSel);
	mon.rstFromWire(rstFromWire);
	mon.dataOut(dataOut);
	mon.hostMode(hostMode);
	mon.rstSyncToBusClkOut(rstSyncToBusClkOut);
	mon.rstSyncToUsbClkOut(rstSyncToUsbClkOut);

  sc_trace_file *tf;
  tf = sc_create_vcd_trace_file("trace");

  sc_trace(tf, dataIn, "dataIn");
  sc_trace(tf, address, "address");

  sc_trace(tf, writeEn, "writeEn");
  sc_trace(tf, strobe_i, "strobe_i");
  sc_trace(tf, busClk, "busClk");
  sc_trace(tf, usbClk, "usbClk");

  sc_trace(tf, dataOut, "dataOut");
  sc_trace(tf, hostSlaveMuxSel, "hostSlaveMuxSel");
  sc_trace(tf, hostMode, "hostMode");
  sc_trace(tf, rstFromWire, "rstFromWire");

  sc_trace(tf, rstSyncToBusClkOut, "rstSyncToBusClkOut");
  sc_trace(tf, rstSyncToUsbClkOut, "rstSyncToUsbClkOut");

  sc_start(1000, SC_NS);
  sc_close_vcd_trace_file(tf);

  return 0;
}

