//////////////////////////////////////////////////////////////////////
////                                                              ////
////  Key generation module implementation                        ////
////                                                              ////
////  This file is part of the SystemC DES                        ////
////                                                              ////
////  Description:                                                ////
////  Generate a key from the previous one                        ////
////                                                              ////
////  To Do:                                                      ////
////   - nothing                                                  ////
////                                                              ////
////  Author(s):                                                  ////
////      - Javier Castillo, jcastilo@opencores.org               ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// Format revision  2009-9-3	nannan
//
// $Log: key_gen_H.cpp,v $
// Revision 1.1  2007-10-23 16:21:01  blancn
// *** empty log message ***
//
// Revision 1.3  2004/09/23 10:16:15  jcastillo
// Removed commentary
//
// Revision 1.2  2004/08/30 16:55:54  jcastillo
// Used indent command on C code
//
// Revision 1.1.1.1  2004/07/05 17:31:17  jcastillo
// First import
//

#include "key_gen_H.h"
#include <iostream>

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	6
#define NUM_OUT 7

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

unsigned int key_gen_H::PC2 (unsigned int x, unsigned int y)
{
	static unsigned int PC2_Permutation[6][8] = {
		{24, 27, 20, 6, 14, 10, 3, 22},
		{0, 17, 7, 12, 8, 23, 11, 5},
		{16, 26, 1, 9, 19, 25, 4, 15},
		{54, 43, 36, 29, 49, 40, 48, 30},
		{52, 44, 37, 33, 46, 35, 50, 41},
		{28, 53, 51, 55, 32, 45, 39, 42},
	};
	return PC2_Permutation[x][y];
}

void key_gen_H::key_gen_0 ()
{
	if(!decrypt)
	{
		prev0 = semi_key[27];
		semi_key = semi_key << 1;
		semi_key[0] = prev0;
	}
	non_perm_key_var.range (55, 28) = semi_key;
	semi_key = prev_key_var.range (27, 0);
	if(!decrypt)
	{
		prev0 = semi_key[27];
		semi_key = semi_key << 1;
		semi_key[0] = prev0;
	}
}

void key_gen_H::key_gen_1 ()
{	
	if(!decrypt)
	{
		prev0 = semi_key[27];
		semi_key = semi_key << 1;
		semi_key[0] = prev0;
	}
	else
	{
		prev0 = semi_key[0];
		semi_key = semi_key >> 1;
		semi_key[27] = prev0;
	}
	non_perm_key_var.range (55, 28) = semi_key;
	semi_key = prev_key_var.range (27, 0);
	if(!decrypt)
	{
		prev0 = semi_key[27];
		semi_key = semi_key << 1;
		semi_key[0] = prev0;
	}
	else
	{
		prev0 = semi_key[0];
		semi_key = semi_key >> 1;
		semi_key[27] = prev0;
	}
}

void key_gen_H::key_gen_2 ()
{	
	if(!decrypt)
	{
		prev0 = semi_key[27];
		prev1 = semi_key[26];
		semi_key = semi_key << 2;
		semi_key[1] = prev0;
		semi_key[0] = prev1;
	}
	else
	{
		prev0 = semi_key[0];
		prev1 = semi_key[1];
		semi_key = semi_key >> 2;
		semi_key[26] = prev0;
		semi_key[27] = prev1;
	}
	non_perm_key_var.range (55, 28) = semi_key;
	semi_key = prev_key_var.range (27, 0);
	if(!decrypt)
	{
		prev0 = semi_key[27];
		prev1 = semi_key[26];
		semi_key = semi_key << 2;
		semi_key[1] = prev0;
		semi_key[0] = prev1;
	}
	else
	{
		prev0 = semi_key[0];
		prev1 = semi_key[1];
		semi_key = semi_key >> 2;
		semi_key[26] = prev0;
		semi_key[27] = prev1;
	}
}

void key_gen_H::generate_key ()
{
  prev_key_var = previous_key;
  new_key_var = 0;
  new_key = 0;
  non_perm_key_var = 0;
  non_perm_key = 0;

	semi_key = prev_key_var.range (55, 28);
	if(iteration == 0)
		key_gen_0();
	else if((iteration == 1) || (iteration == 8) || (iteration == 15))
		key_gen_1();
	else 		
		key_gen_2();
	non_perm_key_var.range (27, 0) = semi_key;
	
	non_perm_key = non_perm_key_var;

  //Apply PC_2 permutation
  //Expand the data    
	for(unsigned int i = 0; i < 6; i++)
	{
		for(unsigned int j = 0; j < 8; j++)
			new_key_var[i * 8 + j] = non_perm_key_var[PC2(i, j)]; // fix bug
	}

  new_key = new_key_var;
}

void key_gen_H::run()
{
	//---------------------------
	read_test_input();
	int c = 0;
	//----------------------------
	
	while(true)
	{
		//---------------------------
		previous_key.range(55,48) = test_data_in[c][0];
		previous_key.range(47,32) = test_data_in[c][1];
		previous_key.range(31,16) = test_data_in[c][2];
		previous_key.range(15,0) = test_data_in[c][3];
		iteration = test_data_in[c][4];
		decrypt = test_data_in[c][5];
		//---------------------------
		
		generate_key();
		
		//---------------------------
		assert(((unsigned) (non_perm_key).range(55,48)) == test_data_out[c][0]);
		assert(((unsigned) (non_perm_key).range(47,32)) == test_data_out[c][1]);
		assert(((unsigned) (non_perm_key).range(31,16)) == test_data_out[c][2]);
		assert(((unsigned) (non_perm_key).range(15,0)) == test_data_out[c][3]);
		assert(((unsigned) (new_key).range(47,32)) == test_data_out[c][4]);
		assert(((unsigned) (new_key).range(31,16)) == test_data_out[c][5]);
		assert(((unsigned) (new_key).range(15,0)) == test_data_out[c][6]);
		c++;
		if(c >= 9)
			break;
		//---------------------------
	}
}
