//////////////////////////////////////////////////////////////////////
////                                                              ////
////  AES mixcolum_H module implementation                          ////
////                                                              ////
////  This file is part of the SystemC AES                        ////
////                                                              ////
////  Description:                                                ////
////  Mixcolum stage implementation for AES algorithm             ////
////                                                              ////
////  To Do:                                                      ////
////   - done                                                     ////
////                                                              ////
////  Author(s):                                                  ////
////      - Javier Castillo, jcastilo@opencores.org               ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// Format revision  2009-9-3	nannan
//
// $Log: mixcolum_H.cpp,v $
// Revision 1.1  2007-10-11 20:37:58  blancn
// *** empty log message ***
//
// Revision 1.1  2005/02/14 11:18:31  jcastillo
// Moved
//
// Revision 1.2  2004/08/30 14:44:44  jcastillo
// Code Formater used to give better appearance to SystemC code
//
// Revision 1.1.1.1  2004/07/05 09:46:22  jcastillo
// First import
//

#include "mixcolum_H.h"
#include <iostream>

#define print_debug(x)   {std::cout << __LINE__ <<" "<< #x << " " << (x) << std::endl;} 
//#define print_debug(x)   ;

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	8
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

void mixcolum_H::update()
{
  w1->in = mix_word;
  outx = w1->outx;
  outy = w1->outy;
}

void mixcolum_H::mux()
{
  //print_debug((int)decrypt_i);
  //print_debug(outx);
  //print_debug(outy);
  outmux = decrypt_i ? outy : outx;
}

void mixcolum_H::mixcol()
{
  sc_biguint<128> data_i_var;
  sc_uint<32> aux;
  sc_biguint<128> data_reg_var;

  data_i_var = data_i;
  data_reg_var = data_reg;
  next_data_reg = data_reg;
  next_state = state;

  mix_word = 0;

  next_ready_o = 0;
  next_data_o = data_o_reg;

	switch (state)
  {
    case 0:
      //print_debug((int)start_i);
      if (start_i)
      {
				aux = data_i_var.range(127, 96);
				//print_debug(aux);
				mix_word = aux;
				data_reg_var.range(127, 96) = outmux;
				next_data_reg = data_reg_var;
				next_state = 1;
				//print_debug("next_state = 1");
      }
      break;
    case 1:
      aux = data_i_var.range(95, 64);
      mix_word = aux;
      //print_debug(outmux);
      data_reg_var.range(95, 64) = outmux;
      next_data_reg = data_reg_var;
      next_state = 2;
      //print_debug("next_state = 2");
      break;
    case 2:
      aux = data_i_var.range(63, 32);
      //print_debug(aux);
      mix_word = aux;
      data_reg_var.range(63, 32) = outmux;
      next_data_reg = data_reg_var;
      next_state = 3;
      break;
    case 3:
      aux = data_i_var.range(31, 0);
      mix_word = aux;
      data_reg_var.range(31, 0) = outmux;
      next_data_o = data_reg_var;
      next_ready_o = 1;
      next_state = 0;
      break;
    default:
      break;
  }
}

void mixcolum_H::registers()
{
  print_debug((int)reset); 
  print_debug((int)next_ready_o); 

  if (!reset)
  {
    data_reg = 0;
    state = 0;
    ready_o = 0;
    data_o_reg = 0;
  }
  else
  {
    data_reg = next_data_reg; 
    state = next_state; 
    ready_o = next_ready_o; 
    data_o_reg = next_data_o; 
  }
}

void mixcolum_H::assign_data_o()
{
  data_o = data_o_reg;
}

void mixcolum_H::run()
{
	//---------------------------
	read_test_input();
	int c = 0;
	//----------------------------
	
	w1 = new word_mixcolum_H("w1");
	
	while(true)
	{
		//-------------------------
		//-------------------------
		
		update();
		assign_data_o();
		mux();
		registers();
		mixcol();
		//-------------------------
		//-------------------------
	}
}
