/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************
 
  bios.cpp -- System Bios.
 
  Original Author: Martin Wang, Synopsys, Inc.
 
 *****************************************************************************/
 
/*****************************************************************************
 
  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.
 
      Name, Affiliation, Date: Nicolas Blanc, ETHZ, 2009
  Description of Modification: major revision

      Name, Affiliation, Date:
  Description of Modification:
 
 *****************************************************************************/

#include "systemc.h"
#include "bios_H.h"

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	2
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------


#define BOOT_LENGTH  6

sc_uint<32> bios_H::imemory[BOOT_LENGTH] = 
{
	0x0f000000,
	0x0f000001, 
	0x0f000002, 
	0x0f000003, 
	0x0f000004, 
	0x0f000005
};

void bios_H::run()
{
	//---------------------------
	read_test_input();
	int c = 0;
	//----------------------------
	
	sc_uint<32> address;

	while (true)
	{
		//------------------------------
		cs = test_data_in[c][0];
		addr = test_data_in[c][1];
		std::cout << sc_time_stamp() << "i-" 
			<< cs << " " 
			<< addr << " " 
			<< std::endl;
		//-------------------------------
		
		if(cs == true) 
		{			
			address = addr;
			if (address < BOOT_LENGTH)
			{		// in BOOTING STAGE    
				// Introduce delay needed
				dataout = imemory[address];
				bios_valid = true;
				bios_valid = false;
			}
			else
			{
				bios_valid = false;
			}
		}

		//-----------------------------
		std::cout << sc_time_stamp() << "o-" << bios_valid << " " 
		<< dataout << " " << std::endl;
		if(test_data_out_x[c][0] == 0)
			assert((unsigned)(bios_valid) == test_data_out[c][0]);
		if(test_data_out_x[c][1] == 0)
			assert((unsigned)(dataout) == test_data_out[c][1]);
		c++;
		if(c >= 9)	break;
		//-----------------------------
	}
} // end of run function

