#include <systemc>
#include <iostream>
#include "bios_L.h"

#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	2
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}


SC_MODULE(testt)
{
	sc_out<bool>	cs;
	sc_out<bool>	CLK;
	sc_out<sc_uint<32> >	addr;
	
	sc_in<sc_uint<32> >	dataout;
	sc_in<bool>	bios_valid;
	
	void stimuli()
	{
		std::string fname = "test_input.txt";
		std::string line;
		std::string subline;
		sc_uint<16> val;
		int c, c1;
		
		// initialize test data
		for(int i = 0; i < MAX_ROW; i++)
			for(int j = 0; j < MAX_COL; j++)
			{
				test_data_in[i][j] = 0;
				test_data_out[i][j] = 0;
				test_data_out_x[i][j] = 0;
			}
			
			// read in test data
			fin.open(fname.c_str());	
		assert(fin.is_open());
		
		getline(fin, line);
		c = 0;
		c1 = 0;
		while(true)
		{
			getline(fin, line);
			if(line.size() == 0)	break;
			
			if(line.find("i_x") != std::string::npos)
			{
				line = line.substr(line.find("i") + 3);
				for(int i = 0; i < NUM_IN; i++)
				{
					subline = Trim(line.substr(0, line.find(":")));
					line = line.substr(line.find(":") + 1);
					assert(subline != "x");
					test_data_in[c][i] = atoi(subline.c_str());
					//std::cout << "i-" << test_data_in[c][i] << " " << atoi(subline.c_str());
				}
				c++;
				//std::cout << std::endl;
			}
			else if(line.find("o_x") != std::string::npos)
			{
				line = line.substr(line.find("o") + 3);
				for(int i = 0; i < NUM_OUT; i++)
				{
					subline = Trim(line.substr(0, line.find(":")));
					line = line.substr(line.find(":") + 1);
					if(subline == "x")
					{
						test_data_out_x[c1][i] = 1;
					}
					else
						test_data_out[c1][i] = atoi(subline.c_str());
					// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
				}
				c1++;
				// 			std::cout << std::endl;
			}
			else
			{
				//std::cout << "else: " << line << std::endl;
				//assert(0);
			}
		}
		
		int c2 = 0;
		std::cout << "enter simulation " << std::endl;
		while(true)
		{
			//std::cout << std::endl;
			wait(5, SC_NS);
			//if(c2 >= 0)
			//{
				cs.write(test_data_in[c2][0]);
				addr.write(test_data_in[c2][1]);
				//}
				std::cout << sc_time_stamp() << "i-" << cs.read() << " " 
					<< addr.read() << " " 
					<< std::endl;
				
				// 			std::cout << "input" << std::endl;
				wait(5, SC_NS);
				// 			
				std::cout << sc_time_stamp() << "o-" << bios_valid.read() << " " 
					<< dataout.read() << " " 
					<< std::endl;
				//if(c2 >= 0)
				//{
					if(test_data_out_x[c2][0] == 0)
						assert((unsigned)(bios_valid.read()) == test_data_out[c2][0]);
					if(test_data_out_x[c2][1] == 0)
						assert((unsigned)(dataout.read()) == test_data_out[c2][1]);
					//}
					c2++;
	}
}

void clk_gen()
{
	while(true) {
		CLK.write(false);
		wait(5, SC_NS);
		CLK.write(true);
		wait(5, SC_NS);
	}
}

SC_CTOR(testt)
{
	SC_THREAD(stimuli); 
	SC_THREAD(clk_gen); 
}
};

// here it goes.
int sc_main(int argc, char *argv[])
{
	sc_set_time_resolution(1, SC_NS);
	
	sc_signal<bool>	cs;
	sc_signal<sc_uint<32> >	addr;
	sc_signal<bool>	CLK;
	
	sc_signal<sc_uint<32> >	dataout;
	sc_signal<bool >	bios_valid;
	
	bios_l ins("ins");
	ins.cs(cs);
	ins.addr(addr);
	ins.CLK(CLK);
	ins.dataout(dataout);
	ins.bios_valid(bios_valid);
	
	testt tb("tester");
	tb.cs(cs);
	tb.addr(addr);
	tb.CLK(CLK);
	tb.dataout(dataout);
	tb.bios_valid(bios_valid);
	
	sc_start(120, SC_NS);
	
	return 0;
}
