module control(
		resetin, 
		instruction, 
		instruction_valid, 
		alu_valid, 
		alu_dout, 
		dram_dout,
		dram_rd_valid,
		dram_wr_ack,
		fpu_dout,
		fpu_valid,
		mmx_dout,
		mmx_valid,
		clear_branch,
		display_done,
		pc,
		rfu_data1,
		rfu_data2,
		CLK,
		next_pc,
		branch_valid,
		branch_target_address,
		dram_cs,
		dram_addr,
		dram_datain,
		alu_op,
		dram_we,
		src_A,
		src_B,
		alu_cs,
		fpu_cs,
		mmx_cs,
		pid_valid, 
		pid_data,
		rfu_rst,
		rfu_cs,
		rfu_re,
		rfu_we,
		rfu_addr1,
		rfu_addr2,
		rfu_addr3,
		rfu_data3);
input  resetin;      		// input reset
input [31:0] instruction;		// fetched instruction
input  instruction_valid;// input valid
input  alu_valid;      	// ALU data valid
input [31:0]	alu_dout; 		  // data from ALU     
input [31:0]	dram_dout;     // data from Dcache
input  dram_rd_valid;    // Dcache read data valid
input  dram_wr_ack;      // Dcache write ack
input [31:0]	fpu_dout;      		// data from FPU
input  fpu_valid;      	// FPU data valid
input [31:0]	mmx_dout;      		// data from FPU
input  mmx_valid;      	// data valid from MMX
input  clear_branch;      	// clear outstanding branch
input  display_done;      	// display to monitor done
input [31:0]	pc;      		// program counter from IFU
input [31:0] rfu_data1;       // RFU data1
input [31:0] rfu_data2;       // RFU data2
input CLK;

output  next_pc;      		// next pc ++ ?
output  branch_valid;      	// branch valid signal
output [31:0]  branch_target_address;	// branch target address
output  dram_cs;      	// memory access valid 
output [31:0]  dram_addr;      	// memory physical address
output [31:0]  dram_datain;      	// memory datain
output [7:0]  alu_op;      		// ALU/FPU/MMU Opcode
output  dram_we;      	  // memory write enable
output [31:0]  src_A;			// operand A
output [31:0]  src_B;			// operand B
output  alu_cs;      	// enable ALU
output  fpu_cs;      	// enable FPU
output  mmx_cs;      	  // enable MMU
output  pid_valid;      	  // load process ID
output [31:0]  pid_data;       // process ID value
output rfu_rst;
output rfu_cs;
output rfu_re;
output rfu_we;
output [3:0] rfu_addr1;    // RFU addr1
output [3:0] rfu_addr2;    // RFU addr2
output [3:0] rfu_addr3;    // RFU addr3
output [31:0] rfu_data3;    // RFU data3

reg [31:0] 	instr_tmp = 0;
reg [7:0]	  opcode_tmp = 0;
reg [3:0]   regA_tmp = 0;
reg [3:0]   regB_tmp = 0;
reg [3:0]   regC_tmp = 0;
reg [31:0]  imm_tmp = 0;
reg [11:0]  offset_tmp = 0;
reg [31:0]  label_tmp = 0;
reg [31:0]  longlabel_tmp = 0;

reg [31:0]  lastreg_tmp = 0;
reg [31:0]	srcA_tmp = 0;
reg [31:0]	srcB_tmp = 0;
reg [31:0]	srcC_tmp = 0;

reg  next_pc;      
reg  branch_valid;     
reg [31:0]  branch_target_address;	
reg  dram_cs;       
reg [31:0]  dram_addr;      
reg [31:0]  dram_datain;      
reg [7:0]  alu_op;      
reg  dram_we;       
reg [31:0]  src_A;
reg [31:0]  src_B;
reg  alu_cs;      
reg  fpu_cs;      
reg  mmx_cs;      
reg  pid_valid;      
reg [31:0]  pid_data;
reg rfu_rst;
reg rfu_cs;
reg rfu_re;
reg rfu_we;
reg [3:0] rfu_addr1; 
reg [3:0] rfu_addr2; 
reg [3:0] rfu_addr3; 
reg [31:0] rfu_data3;

initial
begin
instr_tmp = 0;
opcode_tmp = 0;
regA_tmp = 0;
regB_tmp = 0;
regC_tmp = 0;
imm_tmp = 0;
offset_tmp = 0;
label_tmp = 0;
longlabel_tmp = 0;

lastreg_tmp = 0;
srcA_tmp = 0;
srcB_tmp = 0;
srcC_tmp = 0;

branch_valid = 1'b0;
alu_cs = 0;
fpu_cs = 0;
mmx_cs = 0;
#2;
end

always @ (posedge CLK)
begin
	next_pc = 1;
	alu_cs = 0;
	fpu_cs = 0;
	mmx_cs = 0;
	dram_cs = 0;

	if(instruction_valid == 1'b1)
	begin
		next_pc = 0;
		branch_valid = 0;
		
		instr_tmp = instruction;
		opcode_tmp = instr_tmp[31:24];
		regC_tmp = instr_tmp[23:20];
		regA_tmp = instr_tmp[19:16];
		regB_tmp = instr_tmp[15:12];
		imm_tmp = 0;
    imm_tmp[15:0] = instr_tmp[15:0];
    offset_tmp = instr_tmp[11:0];
    label_tmp[15:0] = instr_tmp[15:0];
    longlabel_tmp = instr_tmp[23:0];

    srcA_tmp = 32'hdeadbeef;
    srcB_tmp = 32'hdeadbeef;
    srcC_tmp = 32'hdeadbeef;
		
		// get values from RFU
		case (opcode_tmp)
			8'h4e, 8'h10, 8'h11, 8'h12, 8'h13, 8'h14, 8'h15:
			begin
				rfu_cs = 1;
				rfu_re = 1;
				rfu_addr1 = regC_tmp;
				rfu_addr2 = regC_tmp;
				#1;
				rfu_cs = 0;
				rfu_re = 0;
				#1;
				srcC_tmp = rfu_data1;
				srcA_tmp = rfu_addr2;
			end
			8'h18:
			begin
				rfu_cs = 1;
				rfu_re = 1;
				rfu_addr1 = 15;
				#1;
				rfu_cs = 0;
				rfu_re = 0;
				#1;
				srcA_tmp = rfu_data1;
				srcB_tmp = rfu_data1;
				srcC_tmp = rfu_addr1;
			end
			default:
			begin
				rfu_cs = 1;
				rfu_re = 1;
				rfu_addr1 = regA_tmp;
				rfu_addr2 = regB_tmp;
				#1;
				rfu_cs = 0;
				rfu_re = 0;
				#1;
				srcA_tmp = rfu_data1;
				srcB_tmp = rfu_data2;
			end
		endcase	

		// Execute operation
		case (opcode_tmp)
			8'h0:
			begin
				$display("halt");
				#2;
			end
			8'h01:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 3;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h02:
			begin
				src_A = srcA_tmp;
				src_B = imm_tmp;
				alu_op = 3;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h03:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 1;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h04:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 4;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h05:
			begin
				src_A = srcA_tmp;
				src_B = imm_tmp;
				alu_op = 4;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h06:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 2;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h07:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 5;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h08:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 6;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h09:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 8;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h0b:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 9;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h0c:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 10;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h0d:
			begin
				src_A = srcA_tmp;
				src_B = 0;
				alu_op = 11;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h0e:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 14;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h4d:
			begin
				dram_cs = 1;
				dram_we = 0;
				offset_tmp = offset_tmp + srcA_tmp;
				dram_addr = offset_tmp;
				alu_op = 14;
				alu_cs = 1;
				wait(dram_rd_valid == 1) begin
					dram_cs = 0;
					srcC_tmp = dram_dout;
					#1;
				end
			end
			8'h4e:
			begin
				dram_cs = 1;
				dram_we = 1;
				offset_tmp = offset_tmp + srcA_tmp;
				dram_addr = offset_tmp;
				dram_datain = srcC_tmp;
				#1;
				dram_cs = 0;
				dram_we = 0;
				#3;
			end
			8'h0f:
			begin
				src_A = srcA_tmp;
				src_B = 0;
				alu_op = 3;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'h10:
			begin
				if(srcC_tmp == srcA_tmp)
				begin
					branch_target_address = pc + label_tmp;
					branch_valid = 1;
				end
				#1;
			end
			8'h11:
			begin
				if(srcC_tmp != srcA_tmp)
				begin
					branch_target_address = pc + label_tmp;
					branch_valid = 1;
				end
				#1;
			end
			8'h12:
			begin
				if(srcC_tmp > srcA_tmp)
				begin
					branch_target_address = pc + label_tmp;
					branch_valid = 1;
				end
				#1;
			end
			8'h13:
			begin
				if(srcC_tmp >= srcA_tmp)
				begin
					branch_target_address = pc + label_tmp;
					branch_valid = 1;
				end
				#1;
			end
			8'h14, 8'h15:
			begin
				if(srcC_tmp < srcA_tmp)
				begin
					branch_target_address = pc + label_tmp;
					branch_valid = 1;
				end
				#1;
			end
			8'h16, 8'h17:
			begin
				branch_target_address = longlabel_tmp;
				branch_valid = 1;
				#1;
			end
			8'h18:
			begin
				branch_target_address = 0;
				branch_target_address = srcC_tmp;
				branch_valid = 1;
				#1;
			end
			8'h29:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 3;
				fpu_cs = 1;
				wait(fpu_valid == 1) begin
					fpu_cs = 0;
					srcC_tmp = fpu_dout;
					#1;
				end
			end
			8'h2a:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 4;
				fpu_cs = 1;
				wait(fpu_valid == 1) begin
					fpu_cs = 0;
					srcC_tmp = fpu_dout;
					#1;
				end
			end
			8'h2b:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 5;
				fpu_cs = 1;
				wait(fpu_valid == 1) begin
					fpu_cs = 0;
					srcC_tmp = fpu_dout;
					#1;
				end
			end
			8'h2c:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 6;
				fpu_cs = 1;
				wait(fpu_valid == 1) begin
					fpu_cs = 0;
					srcC_tmp = fpu_dout;
					#1;
				end
			end
			8'h31:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 3;
				mmx_cs = 1;
				wait(fpu_valid == 1) begin
					mmx_cs = 0;
					srcC_tmp = mmx_dout;
					#1;
				end
			end
			8'h32:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 4;
				mmx_cs = 1;
				wait(fpu_valid == 1) begin
					mmx_cs = 0;
					srcC_tmp = mmx_dout;
					#1;
				end
			end
			8'h33:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 5;
				mmx_cs = 1;
				wait(fpu_valid == 1) begin
					mmx_cs = 0;
					srcC_tmp = mmx_dout;
					#1;
				end
			end
			8'h34:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 6;
				mmx_cs = 1;
				wait(fpu_valid == 1) begin
					mmx_cs = 0;
					srcC_tmp = mmx_dout;
					#1;
				end
			end
			8'h35:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 7;
				mmx_cs = 1;
				wait(fpu_valid == 1) begin
					mmx_cs = 0;
					srcC_tmp = mmx_dout;
					#1;
				end
			end
			8'h36:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 8;
				mmx_cs = 1;
				wait(fpu_valid == 1) begin
					mmx_cs = 0;
					srcC_tmp = mmx_dout;
					#1;
				end
			end
			8'h37:
			begin
				src_A = srcA_tmp;
				src_B = srcB_tmp;
				alu_op = 9;
				mmx_cs = 1;
				wait(fpu_valid == 1) begin
					mmx_cs = 0;
					srcC_tmp = mmx_dout;
					#1;
				end
			end
			8'he0:
			begin
				src_A = 0;
				src_B = 0;
				alu_op = 0;
				rfu_rst = 1;
				#1;
				rfu_rst = 0;
				#1;
			end
			8'hf0:
			begin
				pid_data = longlabel_tmp;
				#1;
				pid_valid = 0;
				#1;
			end
			8'hf1:
			begin
				src_A = imm_tmp;
				src_B = 0;
				alu_op = 3;
				alu_cs = 1;
				wait(alu_valid == 1) begin
					alu_cs = 0;
					srcC_tmp = alu_dout;
					#1;
				end
			end
			8'hff:
			begin
				wait(1) #1;
			end
			default:
			begin
				#1;
			end
		endcase

	// writeback
		case (opcode_tmp)
			8'h01,8'h02,8'h03,8'h04,8'h05,8'h06,8'h07,8'h08,8'h09,
			8'h0b,8'h0c,8'h0d,8'h0e,8'h4d,8'h0f,8'h29,8'h2a,8'h2b,
			8'h2c,8'h31,8'h32,8'h33,8'h34,8'h35,8'h36,8'h37,8'hf1:
			begin
				rfu_cs = 1;
				rfu_we = 1;
				rfu_addr3 = regC_tmp;
				rfu_data3 = srcC_tmp;
				#1;
				rfu_cs = 0;
				rfu_we = 0;
				#1;
			end
			8'h17:
			begin
				rfu_cs = 1;
				rfu_we = 1;
				rfu_addr3 = 15;
				rfu_data3 = pc + 1;
				#1;
				rfu_cs = 0;
				rfu_we = 0;
				#1;
			end
			default:
			begin
				#2;
			end
		endcase
	end 
end
endmodule
















