module exec(reset, cs, opcode, dina, dinb, C, V, Z, dout, out_valid, CLK);
input reset;
input cs;
input [7:0] opcode;
input [31:0] dina;
input [31:0] dinb;
input CLK;

output C;
output V;
output Z;
output [31:0] dout;
output out_valid;

reg C;
reg V;
reg Z;
reg [31:0] dout;
reg out_valid;

reg [31:0] opcode_tmp;
reg [31:0] add1_tmp;
reg [31:0] dina_tmp;
reg [31:0] dinb_tmp;
reg [31:0] dout_tmp;

always @(posedge CLK)
begin
	#3;
	if(cs == 1)
	begin
		dina_tmp = dina;
		dinb_tmp = dinb;
		opcode_tmp = opcode;
		
		// output MUX
		case (opcode_tmp)
			0:	//Stall
			begin
				dout_tmp = dout_tmp;
				#1;	
			end
			1:	//add with carry
			begin
				dout_tmp = dina_tmp + dinb_tmp + add1_tmp;
				#1;	
			end
			2:	//sub with carry
			begin
				dout_tmp = dina_tmp - dinb_tmp - add1_tmp;
				#1;	
			end
			3:	//add without carry
			begin
				dout_tmp = dina_tmp + dinb_tmp;
				#1;	
			end
			4:	//sub without carry
			begin
				dout_tmp = dina_tmp - dinb_tmp;
				#1;	
			end
			5:	//multiply assume 2 clock cycle multiplication
			begin
				dout_tmp = dina_tmp * dinb_tmp;
				#2;	
			end
			6:	//divide assume 2 clock cycle multiplication
			begin
				if(dinb_tmp == 0)
					$display("Division exception - Divide by zero");
				else
					dout_tmp = dina_tmp / dinb_tmp;
				#2;	
			end
			7:	//bitwise NAND
			begin
				dout_tmp = ~(dina_tmp & dinb_tmp);
				#1;	
			end
			8:	//bitwise AND
			begin
				dout_tmp = dina_tmp & dinb_tmp;
				#1;	
			end
			9:	//bitwise OR
			begin
				dout_tmp = dina_tmp | dinb_tmp;
				#1;	
			end
			10:	//bitwise XOR
			begin
				dout_tmp = dina_tmp ^ dinb_tmp;
				#1;	
			end
			11:	//bitwise complement
			begin
				dout_tmp = ~dina_tmp;
				#1;	
			end
			12:	//left shift
			begin
				dout_tmp = dina_tmp << dinb_tmp;
				#1;	
			end
			13:	//right shift
			begin
				dout_tmp = dina_tmp >> dinb_tmp;
				#1;	
			end
			14:	//modulo
			begin
				dout_tmp = dina_tmp % dinb_tmp;
				#1;	
			end
			default:	
			begin
				$display("Bad Opcode");
			end
		endcase

		dout = dout_tmp;
		out_valid = 1;

		if(dout_tmp == 0)
			Z = 1;
		else
			Z = 0;

		if(dout_tmp > 2^32)
			V = 1;
		else
			V = 0;
		#1;
		out_valid = 0;
		#1;
	end
	else
		#1;
end 
endmodule
