/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************
 
  exec.cpp -- Integer Execution Unit.
 
  Original Author: Martin Wang, Synopsys, Inc.
 
 *****************************************************************************/
 
/*****************************************************************************
 
  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.
 
      Name, Affiliation, Date:
  Description of Modification:
 
 *****************************************************************************/


#include "systemc.h"
#include "exec_H.h"

//--------------------------------
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	8
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
	fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			// 			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}
//--------------------------------

void exec_H::run()
{
	//---------------------------
	read_test_input();
	int c = 0;
	//----------------------------
	
	while(1) 
	{
		//---------------------------
		//--------------------------
		
		run_exec();
		
		//---------------------------
		c++;
		if(c >= 9)	break;
		//--------------------------
  }
}

void exec_H::run_exec()
{
	static sc_int<32>			      opcode_tmp = 0;
	static sc_int<32>				    add1_tmp = 0;
	static sc_int<32>		        dina_tmp = 0;
	static sc_int<32>		        dinb_tmp = 0;
	static sc_int<32>		        dout_tmp = 0;
	//
	// main loop
	//
	// initialization of output
	if (cs == true) 
	{
		dina_tmp = dina;
		dinb_tmp = dinb;
		opcode_tmp = opcode;
		
		// output MUX
		switch (opcode_tmp) 
		{
			case 0:         // Stall 
				dout_tmp = dout_tmp;
				break;
			case 1: 	// add with carry
				dout_tmp = dina_tmp + dinb_tmp + add1_tmp;
				break;
			case 2: 	// sub with carry
				dout_tmp = dina_tmp - dinb_tmp - add1_tmp;
				break;
			case 3:         // add without carry
				dout_tmp = dina_tmp + dinb_tmp;
				break;
			case 4:         // sub without carry
				dout_tmp = dina_tmp - dinb_tmp;
				break;
			case 5:         // multiply assume 2 clock cycle multiplication
				dout_tmp = dina_tmp * dinb_tmp;
				break;
			case 6:         // divide assume 2 clock cycle multiplication
				if (dinb_tmp == 0) 
				{
					std::cout << "Division Exception - Divide by zero \n";
				} 
				else 
				{
					dout_tmp = dina_tmp / dinb_tmp;
				}
				break;
			case 7:         // bitwise NAND
				dout_tmp = ~(dina_tmp & dinb_tmp);
				break;
			case 8:         // bitwise AND 
				dout_tmp = dina_tmp & dinb_tmp;
				break;
			case 9:         // bitwise OR
				dout_tmp = dina_tmp | dinb_tmp;
				break;
			case 10:         // bitwise XOR
				dout_tmp = dina_tmp ^ dinb_tmp;
				break;
			case 11:         // bitwise complement
				dout_tmp = ~ dina_tmp;
				break;
			case 12:         // left shift
				dout_tmp = dina_tmp << dinb_tmp;
				break;
			case 13:         // right shift
				dout_tmp = dina_tmp >> dinb_tmp;
				break;
			case 14: 	// modulo
				dout_tmp = dina_tmp % dinb_tmp;
				break;
			default:
				std::cout << "ALU: Bad Opcode (" << opcode_tmp << ")\n";
				break;
		}
		
		dout = dout_tmp;
		out_valid = true;
		
		if (dout_tmp == 0) 
		{
			Z = true;
		} 
		else 
		{
			Z = false;
		}
		if (dout_tmp > 0xffffffff) 
		{
			V = true;
		}
		else 
		{
			V = false;
		}
		out_valid = false;
	} 
	else 
	{}
}
