/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************
 
  fetch.cpp -- Instruction Fetch Unit.
 
  Original Author: Martin Wang, Synopsys, Inc.
 
 *****************************************************************************/
 
/*****************************************************************************
 
  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.
 
      Name, Affiliation, Date:
  Description of Modification:
 
 *****************************************************************************/
 

#include "systemc.h"

struct fetch_h: sc_module 
{ 
  sc_uint<32> 	ramdata;	// instruction from RAM    
  sc_uint<32> 	branch_address; // branch target address   
  bool  			  next_pc;   	// pc ++
  bool  			  branch_valid;  	// branch_valid
  bool  			  interrupt;   	// interrrupt 
  sc_uint<32>   int_vectno;   	// interrupt vector number
  bool  			  ram_valid;   	// ram valid
  bool        	ram_cs;     // RAM chip select
  sc_uint<32>   address;      // address send to RAM
  sc_uint<32>  	instruction; 	 // instruction send to ID
  bool  		  	instruction_valid; // inst valid
  sc_uint<32>   program_counter; // program counter	
  bool  		  	interrupt_ack;   // interrupt acknowledge
  bool  			  branch_clear;    // clear outstanding branch
  bool	  		  reset;  	// reset

  int memory_latency;

  //Constructor 
  SC_CTOR(fetch_h) {
    SC_THREAD(run);
  }
  // Process functionality in member function below
  void run();
};

void fetch_h::run()
{
  sc_uint<32> addr_tmp=0;
  sc_uint<32> datao_tmp=0;
  sc_uint<32> datai_tmp=0;

  addr_tmp = 0;
  datao_tmp = 0xdeadbeef;

  // Now booting from default values
  reset =  true;
  ram_cs =  true;
  address =  addr_tmp;

  do {} while (ram_valid == false);
  
  instruction_valid =  true;
  instruction =  datai_tmp;
  program_counter =  addr_tmp;
  ram_cs =  false;
  instruction_valid =  false;
  addr_tmp++;

  while (true)
  {
    if (addr_tmp == 5)
    {
      reset =  false;
    }

    if (interrupt == true)
    {
      ram_cs =  true;
      addr_tmp = int_vectno;
      datai_tmp = ramdata;

      instruction_valid =  true;
      instruction =  datai_tmp;
      ram_cs =  false;
      interrupt_ack =  true;
      if (next_pc == true) { addr_tmp++; }
      instruction_valid =  false;
      interrupt_ack =  false;
    }

    if(branch_valid == true)
    {
      addr_tmp = branch_address;
    }
    else
    {
      addr_tmp += 1;
    }

    ram_cs =  true;
    address =  addr_tmp;
    //std::cout << "FETCH address " << addr_tmp << "\n";
    do {} while (ram_valid == false);
    datai_tmp = ramdata;

    instruction_valid =  true;
    instruction =  datai_tmp;
    program_counter =  addr_tmp;
    ram_cs =  false;
    instruction_valid =  false;
    do { } while(next_pc.read() == false);
  }
} // end of run function

