module mmxu(cs, opcode, dina, dinb, dout, out_valid, CLK);
input cs;
input [7:0] opcode;
input [31:0] dina;
input [31:0] dinb;
input CLK;

output [31:0] dout;
output out_valid;

reg [7:0]	opcode_tmp;
reg [31:0] 	dout_tmp;

reg [31:0] 	dina_tmp;
reg [31:0] 	dinb_tmp;

reg [31:0] 			dina0_tmp;
reg [31:0] 			dina1_tmp;
reg [31:0] 			dina2_tmp;
reg [31:0] 			dina3_tmp;
reg [31:0] 			dinb0_tmp;
reg [31:0] 			dinb1_tmp;
reg [31:0] 			dinb2_tmp;
reg [31:0] 			dinb3_tmp;
reg [31:0] 			mmxc0_tmp;
reg [31:0] 			mmxc1_tmp;
reg [31:0] 			mmxc2_tmp;
reg [31:0] 			mmxc3_tmp;
reg [31:0] 			mmxcU_tmp;
reg [31:0] 			mmxcL_tmp;

reg [31:0] dout;
reg out_valid;

initial
begin
	opcode_tmp = 0;
	dout_tmp = 0;

	dina_tmp = 0;
	dinb_tmp = 0;

	dina0_tmp = 0;
	dina1_tmp = 0;
	dina2_tmp = 0;
	dina3_tmp = 0;
	dinb0_tmp = 0;
	dinb1_tmp = 0;
	dinb2_tmp = 0;
	dinb3_tmp = 0;
	mmxc0_tmp = 0;
	mmxc1_tmp = 0;
	mmxc2_tmp = 0;
	mmxc3_tmp = 0;
	mmxcU_tmp = 0;
	mmxcL_tmp = 0;
	#3;
end

always@(posedge CLK)
begin
	if(cs == 1)
	begin
		dina_tmp = dina;
		dinb_tmp = dinb;

		dina0_tmp = (dina_tmp & 32'h000000ff);        
		dinb0_tmp = (dinb_tmp & 32'h000000ff);        
		dina1_tmp = (dina_tmp & 32'h0000ff00) >> 8;        
		dinb1_tmp = (dinb_tmp & 32'h0000ff00) >> 8;        
		dina2_tmp = (dina_tmp & 32'h00ff0000) >> 16;        
		dinb2_tmp = (dinb_tmp & 32'h00ff0000) >> 16;        
		dina3_tmp = (dina_tmp & 32'hff000000) >> 24;        
		dinb3_tmp = (dinb_tmp & 32'hff000000) >> 24;        
		opcode_tmp = opcode;

		// output MUX
		case(opcode_tmp)
			0:	//Stall
			begin
				dout_tmp = dout_tmp;
				#1;
			end
			3:	//add
			begin
				mmxc3_tmp = dina3_tmp + dinb3_tmp;
				mmxc2_tmp = dina2_tmp + dinb2_tmp;
				mmxc1_tmp = dina1_tmp + dinb1_tmp;
				mmxc0_tmp = dina0_tmp + dinb0_tmp;
				mmxc3_tmp = (mmxc3_tmp << 24) & 32'hff000000;
				mmxc2_tmp = (mmxc2_tmp << 16) & 32'h00ff0000;
				mmxc1_tmp = (mmxc1_tmp << 8)  & 32'h0000ff00;
				dout_tmp  = mmxc0_tmp | mmxc1_tmp | mmxc2_tmp | mmxc3_tmp; 
				#1;
			end
			4: // add with saturation
			begin
				mmxc3_tmp = dina3_tmp + dinb3_tmp;
				mmxc2_tmp = dina2_tmp + dinb2_tmp;
				mmxc1_tmp = dina1_tmp + dinb1_tmp;
				mmxc0_tmp = dina0_tmp + dinb0_tmp;
				if (mmxc3_tmp >= 256) 
					mmxc3_tmp = 32'hff;
				if (mmxc2_tmp >= 256) 
					mmxc2_tmp = 32'hff;
				if (mmxc1_tmp >= 256) 
					mmxc1_tmp = 32'hff;
				if (mmxc0_tmp >= 256) 
					mmxc0_tmp = 32'hff;

				mmxc3_tmp = (mmxc3_tmp << 24) & 32'hff000000;
				mmxc2_tmp = (mmxc2_tmp << 16) & 32'h00ff0000;
				mmxc1_tmp = (mmxc1_tmp << 8)  & 32'h0000ff00;
				dout_tmp  = mmxc0_tmp | mmxc1_tmp | mmxc2_tmp | mmxc3_tmp;
				#1;
			end
			5: //sub
			begin
				mmxc3_tmp = dina3_tmp - dinb3_tmp;
				mmxc2_tmp = dina2_tmp - dinb2_tmp;
				mmxc1_tmp = dina1_tmp - dinb1_tmp;
				mmxc0_tmp = dina0_tmp - dinb0_tmp;
				mmxc3_tmp = (mmxc3_tmp << 24) & 32'hff000000;
				mmxc2_tmp = (mmxc2_tmp << 16) & 32'h00ff0000;
				mmxc1_tmp = (mmxc1_tmp << 8)  & 32'h0000ff00;
				dout_tmp  = mmxc0_tmp | mmxc1_tmp | mmxc2_tmp | mmxc3_tmp;
				#1;
			end
			6: //sub with saturation
			begin
				mmxc3_tmp = dina3_tmp - dinb3_tmp;
				mmxc2_tmp = dina2_tmp - dinb2_tmp;
				mmxc1_tmp = dina1_tmp - dinb1_tmp;
				mmxc0_tmp = dina0_tmp - dinb0_tmp;
				if (dinb3_tmp > dina3_tmp) mmxc3_tmp = 32'h00;
				if (dinb2_tmp > dina2_tmp) mmxc2_tmp = 32'h00;
				if (dinb1_tmp > dina1_tmp) mmxc1_tmp = 32'h00;
				if (dinb0_tmp > dina0_tmp) mmxc0_tmp = 32'h00;
				mmxc3_tmp = (mmxc3_tmp << 24) & 32'hff000000;
				mmxc2_tmp = (mmxc2_tmp << 16) & 32'h00ff0000;
				mmxc1_tmp = (mmxc1_tmp << 8)  & 32'h0000ff00;
				dout_tmp  = mmxc0_tmp | mmxc1_tmp | mmxc2_tmp | mmxc3_tmp;
				#1;
			end
			7: // packed multiply add with saturation
			begin
				mmxc3_tmp = dina3_tmp * dinb3_tmp;
				mmxc2_tmp = dina2_tmp * dinb2_tmp;
				mmxc1_tmp = dina1_tmp * dinb1_tmp;
				mmxc0_tmp = dina0_tmp * dinb0_tmp;

				if (mmxc3_tmp >= 256) 
					mmxc3_tmp = 32'hff;
				if (mmxc2_tmp >= 256) 
					mmxc2_tmp = 32'hff;
				if (mmxc1_tmp >= 256) 
					mmxc1_tmp = 32'hff;
				if (mmxc0_tmp >= 256) 
					mmxc0_tmp = 32'hff;

				mmxcU_tmp = mmxc3_tmp + mmxc2_tmp;
				mmxcL_tmp = mmxc1_tmp + mmxc0_tmp;
				if (mmxcU_tmp >= 256) 
					mmxcU_tmp = 32'hff;
				if (mmxcL_tmp >= 256) 
					mmxcL_tmp = 32'hff;

				mmxcU_tmp = (mmxcU_tmp << 16) ;
				dout_tmp  = mmxcU_tmp | mmxcL_tmp; 
				#1;
			end
			8: // packed multiply add with saturation
			begin
				mmxc3_tmp = dinb2_tmp << 24; 
				mmxc2_tmp = dinb0_tmp << 16; 
				mmxc1_tmp = dina2_tmp << 8; 
				mmxc0_tmp = dina0_tmp ; 
				dout_tmp  = mmxc3_tmp | mmxc2_tmp | mmxc1_tmp | mmxc0_tmp;
				#1;
			end
			9: // packed multiply add with saturation
			begin
				if (dina3_tmp == dinb3_tmp) 		
					mmxc3_tmp = 32'hff;
				else 	
					mmxc3_tmp = 32'h00;
				if (dina2_tmp == dinb2_tmp) 
					mmxc2_tmp = 32'hff;
				else 
					mmxc2_tmp = 32'h00;
				if (dina1_tmp == dinb1_tmp) 
					mmxc1_tmp = 32'hff;
				else 
					mmxc1_tmp = 32'h00;
				if (dina0_tmp == dinb0_tmp) 
					mmxc0_tmp = 32'hff;
				else 
					mmxc0_tmp = 32'h00;

				mmxc3_tmp = mmxc3_tmp << 24;
				mmxc2_tmp = mmxc2_tmp << 16;
				mmxc1_tmp = mmxc1_tmp << 8;
				mmxc0_tmp = mmxc0_tmp ;
				dout_tmp  = mmxc3_tmp | mmxc2_tmp | mmxc1_tmp | mmxc0_tmp;
				#1;
			end
			default:
				begin
					#1;
				end
		endcase	

		dout = dout_tmp;
		out_valid = 1;

		#1;
		out_valid = 0;
		#1;
	end
	else
	begin
		#1;
	end
end
endmodule