/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************
 
  paging.cpp -- Instruction Paging Unit.
 
  Original Author: Martin Wang, Synopsys, Inc.
 
 *****************************************************************************/
 
/*****************************************************************************
 
  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.
 
      Name, Affiliation, Date: Nicolas Blanc, ETHZ, 2009
  Description of Modification: various code updates

      Name, Affiliation, Date:
  Description of Modification:
 
 *****************************************************************************/


#include "systemc.h"
#include <iostream>
using namespace std;

struct paging_h : sc_module 
{
	bool              csin;              // chip select        
	sc_uint<32>       logical_address; 	// logical address

	sc_uint<32>       icache_din;        // data from icache
	bool              icache_validin;    // data valid bit 

	sc_uint<32>       bios_din;      	   // data from bios
	bool              bios_validin;       // data valid bit 

	sc_uint<32>      dout;              // output data
	bool  	         icache_csout;      // output cs to cache
	bool  	         bios_csout;        // output cs to cache
	sc_uint<32>      physical_address;  // physical address 
	bool  	         data_valid;        // data valid

	signed int pid_reg;             //CPU process ID register      

	SC_CTOR(paging_h) {
		SC_THREAD(run);
		pid_reg = 0;
	}
  void run();
};

void paging_h::entry()
{
  int		address=0;

  while (true)
  {
    do {} while ( !(csin == true) ); 
    address = logical_address;

    if (address >= 5)  // use ICACHE
    {
      physical_address = logical_address;
      icache_csout = true;
      do {} while ( !(icache_validin == true) );
      dout =  icache_din;
      data_valid = true;
      icache_csout = false;
      data_valid = false;
    }
    else              // use BIOS
    {
      physical_address =  logical_address;
      bios_csout = true;
      do {} while ( !(bios_validin == true) );
      dout = bios_din;
      data_valid = true;
      bios_csout = false;
      data_valid = false;
    }
  }
} // end of run function

