/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2006 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License Version 2.4 (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.systemc.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

/*****************************************************************************

  main.cpp - This file instantiates all processes and ties them together
             with signals.

  Original Author: Rashmi Goswami, Synopsys, Inc.

 *****************************************************************************/

/*****************************************************************************

  MODIFICATION LOG - modifiers, enter your name, affiliation, date and
  changes you are making here.

      Name, Affiliation, Date:
  Description of Modification:

 *****************************************************************************/

#include "systemc.h"
#include "registerfile_L.h"

/////////////////////////////////////
#define MAX_ROW	100
#define MAX_COL	100
#define NUM_IN	8
#define NUM_OUT 2

ifstream fin;
unsigned int test_data_in[MAX_ROW][MAX_COL];
unsigned int test_data_out[MAX_ROW][MAX_COL];
unsigned short test_data_out_x[MAX_ROW][MAX_COL];

std::string Trim(const std::string &s) {
	std::string::size_type a = s.find_first_not_of(" ");
	return s.substr(a);
}

void read_test_input()
{
	std::string fname = "test_input.txt";
	std::string line;
	std::string subline;
	int c, c1;
	
	// initialize test data
	for(int i = 0; i < MAX_ROW; i++)
		for(int j = 0; j < MAX_COL; j++)
		{
			test_data_in[i][j] = 0;
			test_data_out[i][j] = 0;
			test_data_out_x[i][j] = 0;
		}
		
		// read in test data
		fin.open(fname.c_str());	
	assert(fin.is_open());
	
	getline(fin, line);
	c = 0;
	c1 = 0;
	while(true)
	{
		getline(fin, line);
		if(line.size() == 0)	break;
		
		if(line.find("i_x") != std::string::npos)
		{
			line = line.substr(line.find("i") + 3);
			for(int i = 0; i < NUM_IN; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				assert(subline != "x");
				test_data_in[c][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				std::cout << "i-" << test_data_in[c][i] << " ";
			}
			c++;
			std::cout << std::endl;
		}
		else if(line.find("o_x") != std::string::npos)
		{
			line = line.substr(line.find("o") + 3);
			for(int i = 0; i < NUM_OUT; i++)
			{
				subline = Trim(line.substr(0, line.find(":")));
				line = line.substr(line.find(":") + 1);
				if(subline == "x")
				{
					test_data_out_x[c1][i] = 1;
				}
				else
					test_data_out[c1][i] = (unsigned int)strtoul(subline.c_str(), NULL, 0);
				// 				std::cout << "o-" << test_data_out[c1][i] << test_data_out_x[c1][i];
			}
			c1++;
			// 			std::cout << std::endl;
		}
		else
		{
			//	std::cout << "else: " << line << std::endl;
			//	assert(0);
		}
	}
}

/////////////////////////////////
SC_MODULE(source) 
{
	sc_out<bool> rst;
	sc_out<bool> cs;
	sc_out<bool> re;
	sc_out<bool> we;
	sc_out<sc_uint<32> > data3;
	sc_out<sc_uint<4> > addr1;
	sc_out<sc_uint<4> > addr2;
	sc_out<sc_uint<4> > addr3;
	
	sc_in_clk CLK;   
	
	SC_CTOR(source)
	{
// 		SC_CTHREAD(entry, CLK.pos());
		SC_THREAD(entry);
	}
	
	//Process Functionality: in member function below
	void entry();
};

void source::entry()
{ 
	int c = 0;
	while(true)
	{ 
// 		wait(CLK.posedge_event());
		wait(10, SC_NS);
		rst.write(test_data_in[c][0]);
		cs.write(test_data_in[c][1]);
		re.write(test_data_in[c][2]);
		we.write(test_data_in[c][3]);
		addr1.write(test_data_in[c][4]);
		addr2.write(test_data_in[c][5]);
		addr3.write(test_data_in[c][6]);
		data3.write(test_data_in[c][7]);
		
		std::cout << sc_time_stamp() << "i-"
		<< rst.read() << " "
		<< cs.read() << " "
		<< re.read() << " "
		<< we.read() << " "
		<< addr1.read() << " "
		<< addr2.read() << " "
		<< addr3.read() << " "
		<< (unsigned)data3.read() << " "
		<< std::endl;
		c++;
		if(c > 10)	break;
	}
}

SC_MODULE(sink)
{
	sc_in<sc_uint<32> > data1;
	sc_in<sc_uint<32> > data2;

	sc_in_clk CLK;
		
	SC_CTOR(sink)
	{
		SC_CTHREAD(entry, CLK.neg());
	}
	
	void entry(); 
};

void sink::entry()
{
	int c = 0;
	while(true)
	{ 
// 		wait(CLK.negedge_event());
		wait();
		std::cout << sc_time_stamp() << "o-"
			<< (unsigned)data1.read() << " "
			<< (unsigned)data2.read() << " "
			<< std::endl;
		
		if(test_data_out_x[c][0] == 0)
			assert((unsigned)(data1.read()) == test_data_out[c][0]);
		if(test_data_out_x[c][1] == 0)
			assert((unsigned)(data2.read()) == test_data_out[c][1]);
		c++;
	}
}

int sc_main(int ac, char* av[])
{
	read_test_input();
	
	sc_signal<bool> rst;
	sc_signal<bool> cs;
	sc_signal<bool> re;
	sc_signal<bool> we;
	sc_signal<sc_uint<32> > data3;
	sc_signal<sc_uint<4> > addr1;
	sc_signal<sc_uint<4> > addr2;
	sc_signal<sc_uint<4> > addr3;
	
	sc_signal<sc_uint<32> > data1;
	sc_signal<sc_uint<32> > data2;
	
	sc_clock clock("CLOCK", 10, SC_NS, 0.5, 0.0, SC_NS);

	registerfile_L U("registerfile_L"); 
	U.clk(clock);
	U.rst(rst);
	U.cs(cs);
	U.re(re);
	U.we(we);
	U.data3(data3);
	U.addr1(addr1);
	U.addr2(addr2);
	U.addr3(addr3);
	U.data1(data1);
	U.data2(data2);
	
	source SOURCE1("SOURCEPROCESS");
	SOURCE1.rst(rst);
	SOURCE1.cs(cs);
	SOURCE1.re(re);
	SOURCE1.we(we);
	SOURCE1.data3(data3);
	SOURCE1.addr1(addr1);
	SOURCE1.addr2(addr2);
	SOURCE1.addr3(addr3);
	SOURCE1.CLK(clock);
	
	sink SINK1("SINKPROCESS");
	SINK1.data1(data1);
	SINK1.data2(data2);
	SINK1.CLK(clock);

	sc_start(140, SC_NS);
	return 0;
}

