#include "systemc.h"
#include <iostream>
#include "wb_master_model_L.h"

SC_MODULE(testt)
{
	sc_out<bool>   clk;
	sc_out<bool>   rst;

	sc_out<sc_uint<dwidth-1> > din;
	sc_out<bool>   ack;
	sc_out<bool>   err;
	sc_out<bool>   rty;

	void stimuli()
	{
		rst.write(true);

		din.write(100);
		ack.write(false);
		err.write(false);
		rty.write(false);

		wait(10, SC_NS); 
		rst.write(false);

		wait(10, SC_NS); 
		ack.write(true);
		err.write(true);
		rty.write(true);

		wait(10, SC_NS); 
		ack.write(false);

		wait(10, SC_NS); 
		err.write(false);

		wait(10, SC_NS); 
		rty.write(false);
		ack.write(true);

		wait(20, SC_NS); 
		sc_stop();
	}

	void clk_gen()
	{
		while(true) 
		{
			clk.write(false);
			wait(5, SC_NS);
			clk.write(true);
			wait(5, SC_NS);
		}
	}

	SC_CTOR(testt) 
	{
		SC_THREAD(stimuli);
		SC_THREAD(clk_gen);
	}
};

SC_MODULE(monitor)
{
  sc_in<bool>	clk;
  sc_in<bool>	rst;
  sc_in<sc_uint<dwidth   -1> > din;
  sc_in<bool>	ack;
  sc_in<bool>	err;
  sc_in<bool>	rty;
  
  sc_in<sc_lv<awidth   -1> > adr;
  sc_in<sc_lv<dwidth   -1> > dout;
  sc_in<bool>	cyc;
  sc_in<bool>	stb;
  sc_in<bool>	we;
  sc_in<sc_lv<dwidth/8   -1> > sel;

  void MonGen()
  {
    std::cout << sc_time_stamp() << " " << clk << " " << rst << " " << din << " "
	    << ack << " " << err << " " << rty << " \t" 
	    << adr << " " << dout << " " << cyc << " " << stb << " "
	    << we << " " << sel
	    << std::endl;
  }

  SC_CTOR(monitor)
  {
    SC_METHOD(MonGen);
    sensitive << clk.pos();
  }
};

// here it goes.
int sc_main(int argc, char *argv[])
{
  sc_signal<bool>   clk;
  sc_signal<bool>   rst;

  sc_signal<sc_uint<dwidth - 1> > din;
  sc_signal<bool>   ack;
  sc_signal<bool>   err;
  sc_signal<bool>   rty;

  sc_signal<sc_lv<awidth   -1> > adr;
  sc_signal<sc_lv<dwidth   -1> > dout;
  sc_signal<bool>   cyc;
  sc_signal<bool>   stb;
  sc_signal<bool>   we;
  sc_signal<sc_lv<dwidth/8   -1> > sel;

  sc_set_time_resolution(1, SC_NS);

  wb_master_model_L ins("ins");
	ins.clk(clk);
	ins.rst(rst);
	ins.din(din);
	ins.ack(ack);
	ins.err(err);
	ins.rty(rty);
	ins.adr(adr);
	ins.dout(dout);
	ins.cyc(cyc);
	ins.stb(stb);
	ins.we(we);
	ins.sel(sel);

  testt tb("tb");
	tb.clk(clk);
	tb.rst(rst);
	tb.din(din);
	tb.ack(ack);
	tb.err(err);
	tb.rty(rty);

  monitor mon("mon");
	mon.clk(clk);
	mon.rst(rst);
	mon.din(din);
	mon.ack(ack);
	mon.err(err);
	mon.rty(rty);
	mon.adr(adr);
	mon.dout(dout);
	mon.cyc(cyc);
	mon.stb(stb);
	mon.we(we);
	mon.sel(sel);

  sc_trace_file *tf;
  tf = sc_create_vcd_trace_file("trace");

  sc_trace(tf, clk, "clkd");
  sc_trace(tf, rst, "rst");

  sc_trace(tf, din, "din");
  sc_trace(tf, ack, "ack");
  sc_trace(tf, err, "err");
  sc_trace(tf, rty, "rty");

  sc_trace(tf, adr, "adr");
  sc_trace(tf, dout, "dout");
  sc_trace(tf, cyc, "cyc");
  sc_trace(tf, stb, "stb");

  sc_trace(tf, we, "we");
  sc_trace(tf, sel, "sel");

  sc_start(1000, SC_NS);
  sc_close_vcd_trace_file(tf);

  return 0;
}

