#include "systemc.h"

// constants
#define dwidth		32
#define awidth  	32
// added by nan
#define a_nan 		5
#define d_exp_nan	5

SC_MODULE(wb_master_model_H)
{
  bool	rst;
  sc_uint<dwidth-1> din;
  bool	ack;
  bool	err;
  bool	rty;
  
  sc_lv<awidth-1> adr;
  sc_lv<dwidth-1> dout;
  bool	cyc;
  bool	stb;
  bool	we;
  sc_lv<dwidth/8-1> sel;

  void initializeMem();
  void wb_write_task(unsigned, sc_lv<awidth-1>, sc_lv<dwidth-1>);
  sc_lv<dwidth-1> wb_read_task(unsigned, sc_lv<awidth-1>);
  void wb_cmp_task(unsigned, sc_lv<awidth-1>, sc_lv<dwidth-1>);
  // added by nan
  void call_task();
	
  sc_uint<dwidth-1> q;
  unsigned delay_nan;
  sc_lv<dwidth-1> lv_d;
  sc_lv<awidth-1> lv_a;

  SC_CTOR(wb_master_model_H)
  {
    SC_THREAD(initializeMem);
    SC_THREAD(call_task);
  }
};

void wb_master_model_H::initializeMem()
{
  for(unsigned i=0; i < awidth; i++)    lv_a[i]="X";
  for(unsigned i=0; i < dwidth; i++)    lv_d[i]="X";

  adr=lv_a;
  dout=lv_d;
  cyc=false;
  stb="X";
  we="X";
  sel=lv_d.range(dwidth/8 - 1, 0);
  // nannan
  delay_nan=7;
}


void wb_master_model_H::wb_write_task(unsigned delay, sc_lv<awidth -1> a, sc_lv<dwidth -1> d)
{
  adr=a;
  dout=d;
  cyc=true;
  stb=true;
  we=true;

  while(true)
  {
    if(ack == true)
      break;
  }

  for(unsigned i=0; i < dwidth; i++)    lv_d[i]="1";
  sel=lv_d.range(dwidth/8 - 1, 0);

  cyc=false;
  stb="X";
  we="X";

  for(unsigned i=0; i < awidth; i++)    lv_a[i]="X";
  for(unsigned i=0; i < dwidth; i++)    lv_d[i]="X";

  adr=lv_a;
  dout=lv_d;
  sel=lv_d.range(dwidth/8 - 1, 0);
}

sc_lv<dwidth -1>  wb_master_model_H::wb_read_task(unsigned delay, sc_lv<awidth -1> a)
{
  unsigned i=0;
  while(i < delay)
  {
    i++;
  }

  { 
    adr=a;

    for(unsigned i=0; i < dwidth; i++)    lv_d[i]="X";
    dout=lv_d;

    cyc=true;
    stb=true;
    we=true;
  }

  while(true)
  {
    if(ack == true)
      break;
  }

  for(unsigned i=0; i<dwidth; i++)    lv_d[i]="1";
  sel.range(dwidth/8-1, 0);

  cyc=false;
  stb="X";
  we="X";

  for(unsigned i=0; i < awidth; i++)    lv_a[i]="X";
  for(unsigned i=0; i < dwidth; i++)    lv_d[i]="X";

  adr=lv_a;
  dout=lv_d;
  sel=lv_d.range(dwidth/8-1, 0);
  return din;
}

void wb_master_model_H::wb_cmp_task(unsigned delay, sc_lv<awidth-1> a, sc_lv<dwidth-1> d_exp)
{
  q=wb_read_task(delay, a);

  if(d_exp!=q)
  {
    cout << "\n--- ERROR: At address 0x" << a << ",got 0x" << 
      q << ", expected 0x" << d_exp << "at time " << sc_time_stamp() << endl;
    sc_stop();
  }
}

void wb_master_model_H::call_task()
{
  wb_cmp_task(delay_nan, a_nan, d_exp_nan);
}

