#include "systemc.h"

// constants
#define dwidth		32
#define awidth  	32
// added by nan
#define a_nan 		5
#define d_exp_nan	5

SC_MODULE(wb_master_model_L)
{
  sc_in<bool>	clk;
  sc_in<bool>	rst;
  sc_in<sc_uint<dwidth   -1> > din;
  sc_in<bool>	ack;
  sc_in<bool>	err;
  sc_in<bool>	rty;
  
  sc_out<sc_lv<awidth   -1> > adr;
  sc_out<sc_lv<dwidth   -1> > dout;
  sc_out<bool>	cyc;
  sc_out<bool>	stb;
  sc_out<bool>	we;
  sc_out<sc_lv<dwidth/8   -1> > sel;

  void initializeMem();
  void wb_write_task(unsigned, sc_lv<awidth -1>, sc_lv<dwidth -1>);
  sc_lv<dwidth -1> wb_read_task(unsigned, sc_lv<awidth -1>);
  void wb_cmp_task(unsigned, sc_lv<awidth -1>, sc_lv<dwidth -1>);
  // added by nan
  void call_task();
	
  sc_signal<sc_uint<dwidth   -1> > q;
  unsigned delay_nan;
  sc_lv<dwidth   -1> lv_d;
  sc_lv<awidth   -1> lv_a;

  SC_CTOR(wb_master_model_L)
  {
    SC_THREAD(initializeMem);

    SC_METHOD(call_task);
    sensitive << clk.pos();
  }
};


void wb_master_model_L::initializeMem()
{
  for(unsigned i = 0; i < awidth; i++)    lv_a[i] = "X";
  for(unsigned i = 0; i < dwidth; i++)    lv_d[i] = "X";

  adr.write(lv_a);
  dout.write(lv_d);
  cyc.write(false);
  stb.write("X");
  we.write("X");
  sel.write(lv_d.range(dwidth/8 - 1, 0));

  wait(1, SC_NS); 
}

void wb_master_model_L::wb_write_task(unsigned delay, sc_lv<awidth -1> a, sc_lv<dwidth -1> d)
{
  unsigned i = 0;
  while(i < delay)
  {
    i++;
    next_trigger(clk.posedge_event());
  }

  next_trigger(1, SC_NS); 
  if(clk.posedge())
  {
    adr.write(a);
    dout.write(d);
    cyc.write(true);
    stb.write(true);
    we.write(true);
  }

  while(true)
  {
    if(ack.read() == true)
      break;
    next_trigger(clk.posedge_event());
  }

  for(unsigned i = 0; i < dwidth; i++)    lv_d[i] = "1";
  sel.write(lv_d.range(dwidth/8 - 1, 0));

  next_trigger(1, SC_NS); 
  cyc.write(false);
  stb.write("X");
  we.write("X");

  for(unsigned i = 0; i < awidth; i++)    lv_a[i] = "X";
  for(unsigned i = 0; i < dwidth; i++)    lv_d[i] = "X";

  adr.write(lv_a);
  dout.write(lv_d);
  sel.write(lv_d.range(dwidth/8 - 1, 0));
}

sc_lv<dwidth -1>  wb_master_model_L::wb_read_task(unsigned delay, sc_lv<awidth -1> a)
{
  unsigned i = 0;
  while(i < delay)
  {
    i++;
    next_trigger(clk.posedge_event());
  }

  next_trigger(1, SC_NS);

  adr.write(a);
  for(unsigned i = 0; i < dwidth; i++)    lv_d[i] = "X";
  dout.write(lv_d);
  cyc.write(true);
  stb.write(true);
  we.write(true);

  next_trigger(clk.posedge_event());

  if(ack.read() == false)
  {
    for(unsigned i = 0; i < dwidth; i++)    lv_d[i] = "1";
    sel.write(lv_d.range(dwidth/8 - 1, 0));

    next_trigger(1, SC_NS);

    cyc.write(false);
    stb.write("X");
    we.write("X");

    for(unsigned i = 0; i < awidth; i++)    lv_a[i] = "X";
    for(unsigned i = 0; i < dwidth; i++)    lv_d[i] = "X";

    adr.write(lv_a);
    dout.write(lv_d);
    sel.write(lv_d.range(dwidth/8 - 1, 0));
    return din.read();
  }
  else
    return lv_d;
}

void wb_master_model_L::wb_cmp_task(unsigned delay, sc_lv<awidth -1> a, sc_lv<dwidth -1> d_exp)
{
  q.write(wb_read_task(delay, a));

  if(d_exp != q)
  {
    cout << "\n--- ERROR: At address 0x" << a << ",got 0x" << 
      q << ", expected 0x" << d_exp << "at time " << sc_time_stamp() << endl;
    sc_stop();
  }
}

void wb_master_model_L::call_task()
{
  delay_nan = 0;
  wb_cmp_task(delay_nan, a_nan, d_exp_nan);
}

